package RPM::Source::Dependency::Analyzer::Collector;

use strict;
use warnings;

use Cwd;
use Carp;
use File::Path qw(make_path remove_tree);
use File::Temp qw/tempfile tempdir/;
use File::Basename;
use Source::Shared::CLI;
use Source::Bundle;

use RPM::Source::Dependency::Analyzer::FileListSeparator;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Autoconf;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::AutoMake;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::CPP_header;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::CMake;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::QMake;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Configure;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::ShEvalPkgConfig;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Perl;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Python;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Lua;
use RPM::Source::Dependency::Analyzer::Collector::Plugin::Ruby;

our @ISA=qw/Source::Shared::CLI/;

# TODO already ban_filepatterns in PerlBuilder
my ($opt_skip_analyze_by_pattern);

### options for SourceAnalyzer::DependencyCollector;
our $opt_hint='';
our ($opt_analyze_subdir_autoconf);
our $opt_scan_configure_for_broken_cpp_check=0;
our $opt_disable=0;

our $verbose=0;

our @LONGOPT=(
    # TODO: share with resolver
    'sourcedep-disable'   => \$opt_disable,

    'sourcedep-hint=s'   => \$opt_hint,
    'sourcedep-autoconf-search-subdirs!' => \$opt_analyze_subdir_autoconf,
    );


sub print_longopt {
    print "SourceAnalyzer Options:
	--sourcedep-disable - disable SourceAnalyzer.
";
    print "SourceAnalyzer::DependencyCollector Options:
	--sourcedep-hint <hint> - pass a hint to SourceAnalyzer core.
	  current hints are: nodejs octave perl
	--sourcedep-autoconf-search-subdirs	search subdirs for extra configure.ac/in files
	--no-sourcedep-autoconf-search-subdirs  negate the option (default)
";
}

sub new {
    my $class = shift;
    my $self = {
	-verbose => $verbose,
	-hint => $opt_hint,

	# TODO: something like (rebased filelist) '^examples/'
	# already ban_filepatterns
	#-skip_analyze_by_pattern => $opt_skip_analyze_by_pattern,

	-analyze_subdir_autoconf => $opt_analyze_subdir_autoconf,
	-scan_configure_for_broken_cpp_check => $opt_scan_configure_for_broken_cpp_check,

	@_
    };
    bless $self, $class;

    my $hint=$self->{-hint};
    if ($hint and $hint eq 'octave') {
    	$self->{-analyze_subdir_autoconf}=1 if not defined $self->{-analyze_subdir_autoconf};
    }

    return $self;
}

sub collect_source_dependencies {
    my ($self,$storage,$archive_obj)=@_;
    Carp::cluck "analyze: $archive_obj should be an object of SourceArchive" unless ref $archive_obj;
    my $localverbose=$self->{-verbose};
    $localverbose=$verbose if $verbose > $localverbose;

    my $hint=$self->{-hint};
    print STDERR "INFO: DependencyCollector: hint=$hint\n" if $localverbose >2;

    my $root_prefix=$archive_obj->depth_prefix();

    # do not enter git dir
    my @filelist=grep {not m!(?:^|/)(?:\.git|\.svn|CVS)(?:/|$)!} @{$archive_obj->filelist()};
    # TODO: ._ macos (if same name exists), .cvs/git ignore, etc, bzr
    #$ban{$dir.'/.*.patch'} $ban{$dir.'/.*.diff'}


    my $_to_ban=$self->{-ban_filepatterns};
    if ($_to_ban) {
	my $_to_ban_re;
	$_to_ban_re='(?:^|/)('.join('|',@$_to_ban).')';
	@filelist = grep { not m!$_to_ban_re!} @filelist;
	# TODO
	#print STDERR "INFO: DependencyCollector: banned $_\n" if $localverbose >2;
    }

    my $separ=RPM::Source::Dependency::Analyzer::FileListSeparator->new(
	-depth_prefix => $root_prefix,
	-storage => $storage,
	-hint => $hint,
	);
    $separ->separate_filelist(\@filelist);

    my @configureac=$separ->get('configureac');
    if (not $self->{-analyze_subdir_autoconf}) {
	# temporary?; let us analyze root only
	@configureac=grep {not m!^.*/.*/configure!} @configureac;
    }
    if ($hint and $hint eq 'octave') {
	unshift @configureac, $separ->get('configureoctave');
    }

    $storage->set_lang_flag('c') if $separ->has('c_files');
    $storage->set_lang_flag('cxx') if $separ->has('cpp_files');
    $storage->set_lang_flag('fortran') if $separ->has('fortran_files') and $hint and $hint eq 'octave';
    $storage->set_lang_flag('go') if $separ->has('go_files');
    $storage->set_lang_flag('lua') if $separ->has('lua_files');
    $storage->set_lang_flag('ruby') if $separ->has('ruby_files');
    $storage->set_lang_flag('swig') if $separ->has('swig_files');
    $storage->set_lang_flag('texinfo') if $separ->has('texinfo');
    $storage->set_lang_flag('perl_buildscripts') if $separ->has('perl_buildscripts');
    $storage->set_lang_flag('perl_xs') if $separ->has('xs_files');


    ### unpacking and running collectors plugins

    # TODO: must be gathered from plugins used
    my @unpack_list=map {$separ->get($_)} @RPM::Source::Dependency::Analyzer::FileListSeparator::DEFAULT_UNPACK;
    push @unpack_list, $separ->get('raw_configure') if $self->{-scan_configure_for_broken_cpp_check};

    my $unpack_c=0;
    $unpack_c=1 if $hint and ($hint eq 'perl' or $hint eq 'python' or $hint eq 'nodejs' or $hint eq 'lua' or $hint eq 'ruby');
    push @unpack_list, map {$separ->get($_)} qw/c_files cpp_files/ if $unpack_c;

    if ($localverbose>4) {
	if (@unpack_list) {
	    print STDERR "DEBUG: Dependency::Analyzer::Collector: unpack list:\n", map {"- $_\n"} @unpack_list;
	} else {
	    print STDERR "DEBUG: Dependency::Analyzer::Collector: unpack list empty\n";
	}
    }
    if (@unpack_list) {
	# unpack 
	my $unpackdir = $archive_obj->extract(-list=>\@unpack_list);
	if (grep {not -e "$unpackdir/$_"} @unpack_list) {
	    print STDERR "DEBUG: Dependency::Analyzer::Collector: partial unpack is broken. broken files:\n", map {"- $_\n"} grep {not -e "$unpackdir/$_"} @unpack_list if $localverbose>4;

	    # some of unpackef files seems to be (broken) symlinks; no choice, unpack all
	    $unpackdir = $archive_obj->extract();
	}

	my $__parse_with = sub {
	    my ($pclass,@files)=@_;
	    return unless @files;
	    my $parser=$pclass->new(
		-storage => $storage,
		-verbose => $self->{verbose},
		);
	    foreach my $file (@files) {
	    print STDERR "DEBUG: parse_file: $file\n" if $localverbose>3;
		$parser->parse_file("$unpackdir/$file");
	    }
	    $parser->finish();
	};

	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Autoconf',@configureac);

	if ($self->{-scan_configure_for_broken_cpp_check}) {
	    print STDERR "DEBUG: Collector: -scan_configure_for_broken_cpp_check=1, going to parse: [",join(',',$separ->get('raw_configure')),"]\n" if $localverbose>2;
	    &$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Configure',$separ->get('raw_configure'));
	}
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::AutoMake',$separ->get('makefileam'));
	unless (@configureac) {
	    # TODO: add proper /.../qmake (qt3,qt4,qt5)
	    &$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::QMake',$separ->get('qmakepro'));
	    &$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::CMake',$separ->get('cmakelists'));
	    warn "info: SConstruct files found - not supported yet\n" if $separ->get('sconstruct');
	}
	if ($unpack_c) {
	    my @unpack_c=map {$separ->get($_)} qw/c_files cpp_files/;
	    &$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::CPP_header',@unpack_c);
	}
	# todo: add plain Makefile's if no other alternative
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::ShEvalPkgConfig',$separ->get('perl_buildscripts'));

	# todo scripts & magic
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Lua',$separ->get('lua_files'));
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Ruby',$separ->get('ruby_files'));

	#### TODO: rewrite as filter class!
	$self->__analyze_scripts_by_magic($unpackdir, $separ, $hint);
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Perl',$separ->get('perl_files'));
	my @python_f=map {$separ->get($_)} qw/python_files python_scripts/;
	&$__parse_with('RPM::Source::Dependency::Analyzer::Collector::Plugin::Python',@python_f);
    }

    # python ignore
    &__fill_ignore_from_script_files($separ, $storage, 'python_files','python',
	sub {
	    my $module=$_[0];
	    $module=~s!(/__init__)?\.py$!!;
	    $module=~s!/!.!g;
	    return $module;
	});

    # lua ignore
    &__fill_ignore_from_script_files($separ, $storage, 'lua_files','lua',
	sub {
	    my $module=$_[0];
	    $module=~s!\.lua$!!;
	    $module=~s!/!.!g;
	    return $module;
	});

    # ruby ignore
    &__fill_ignore_from_script_files($separ, $storage, 'ruby_files','ruby',
	sub {
	    my $module=$_[0];
	    $module=~s!\.rb$!!;
	    $module=~s!^lib/!!g;
	    return $module;
	});

}

sub __fill_ignore_from_script_files {
    my ($separ, $storage, $separ_key, $storage_key, $mapper_sub)=@_;
    my @script_files=$separ->get($separ_key);
    my $script_depth_ref=&Source::Bundle::_depth_from_filelist(\@script_files);
    my $rebased_script=&Source::Bundle::_rebase_filelist(\@script_files,$script_depth_ref);
    foreach my $scriptfile (@$rebased_script) {
	my $module=&$mapper_sub($scriptfile);
	$storage->add_raw_storage_deps($storage_key,'ignore',$module);
    }
}


### TODO filter class
sub __analyze_scripts_by_magic {
    my ($self, $unpackdir,$separ, $hint)=@_;
    my @inperl;
    my $force_perl_by_extensions = ($hint and $hint eq 'perl');
    my $force_python_by_extensions=1;
    $force_perl_by_extensions=1 if $separ->has('perl_buildscripts');
    push @inperl, map {$separ->get($_)} qw/guess_test_perl perl_buildscripts/ if $force_perl_by_extensions;
    my @guess_perl=$separ->get('guess_perl');
    my @guess_scripts=$separ->get('guess_scripts');
    print STDERR "guess_scripts=",join(',',@guess_scripts),"\n" if $verbose >2;
    my @inpython;
    foreach my $file (@guess_perl, @guess_scripts) {
	# see examples of magic output below
	my $magic = `file -b "$unpackdir/$file"`;
	if ($magic =~ /^[Pp]erl script[, ]/ || $magic =~ /[Pp]erl5 module source/
	    # see examples-magic
	    || ($magic =~ /ASCII text/ and $file =~ /\.(?:pl)$/)
	    # ASCII English text
	    || ($force_perl_by_extensions and ($file =~ /\.(?:p[ml]|PL)$/ or $file =~ m!(?:^|/)t/.+\.t$!))
	    ) {
	    push @inperl,$file;
	} elsif ($magic =~ /^[Pp]ython script[, ]/
	    # copycat from perl above
	    || ($magic =~ /ASCII text/ and $file =~ /\.py$/)
	    # ASCII English text
	    || ($force_python_by_extensions and $file =~ /\.py$/)
	    ) {
	    push @inpython,$file;
	} else {
	    print STDERR "skipped $file: magic=$magic\n" if $self->{-verbose} >2;
	}
    }
    $separ->add('perl_files',@inperl);
    $separ->add('python_scripts',@inpython);
}

1;

__END__

$ file --version
file-4.26
magic file from /etc/magic:/usr/share/file/magic
$ file testdata/python/greenrpc/bin/greenrpc-client
testdata/python/greenrpc/bin/greenrpc-client: python script text executable
$ file lib/RPM/Source/Dependency/Analyzer.pm
lib/RPM/Source/Dependency/Analyzer.pm: Perl5 module source text

$ file --version
file-5.40
magic file from /etc/magic:/usr/share/file/magic
seccomp support included
$ file testdata/python/greenrpc/bin/greenrpc-client
testdata/python/greenrpc/bin/greenrpc-client: Python script, ASCII text executable
$ file lib/RPM/Source/Dependency/Analyzer.pm
lib/RPM/Source/Dependency/Analyzer.pm: Perl5 module source, ASCII text
