package RPM::Source::Dependency::Analyzer::Collector::Parser::Autoconf;

use strict;
use warnings;
use Carp;
use RPM::Source::Dependency::Analyzer::Collector::Parser;

our $VERSION=0.001;
our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser/;

# for copypaste from Text/ParseWords.pm; seems to be support for perlish syntax not useful for us.
my $PERL_SINGLE_QUOTE;
our $verbose=0;

sub new {
    my $class = shift;
    my $self = {
	-vars => {},
	-verbose => 0,
	-varhook => \&_var_handler,
	@_
    };
    bless $self, $class;
}

sub parse_string {
    my ($self,$data)=@_;
    $self->{-vardata}={};
    my $pure_data='';
    while (length($data)) {
	&__purge_dnl(\$pure_data,\$data);
    }
    while (length($pure_data)) {
	&_read_any($self,\$pure_data);
    }
}

# C++ features
#CXXAC_TYPE_BOOL
#CXXAC_BAD_ALLOC
#CXXAC_HAVE_EXPLICIT
#CXXAC_HAVE_MUTABLE
#CXXAC_HAVE_ARRAY_NEW
#CXXAC_BAD_DELETE
#CXXAC_DEFAULT_EXCEPTION
#CXXAC_HAVE_TYPENAME
#CXXAC_NO_GUIDE_DECL
#CXXAC_HEADER_ALL

my %KNOWN_MACRO=map {$_=>1} qw!
DAP_CHECK_ZLIB
DECAF_CONFIGURE_APR
DECAF_CONFIGURE_SSL
GNOME_COMMON_INIT
GNOME_COMPILE_WARNINGS
GNOME_DEBUG_CHECK
GNOME_DOC_INIT
PKG_CHECK_MODULES
PKG_CHECK_EXISTS
PKG_PROG_PKG_CONFIG
GLIB_GSETTINGS
GTK_DOC_CHECK
AST_IMLIB2_SUPPORT
JH_CHECK_XML_CATALOG
!;

sub __purge_dnl {
    my ($outref,$inref)=@_;
    my $read='';
    if ($$inref=~s/^([^\n]*\n)//) {
	$read=$1;
    } else {
	$read=$$inref;
	$$inref='';
    }
    $read=~s/\b(?:[mM]4_)?[Dd][Nn][Ll].*$//s;
    $$outref .= $read;
}

sub _read_any {
    my ($self,$ref)=@_;
    my $read=&_read_non_token_non_end_or_comments($ref);
    return $read if $read ne '';
    $read=$1 if $$ref=~s/^([,\)])//;
    return $read if $read ne '';
    my $token=&_read_token($ref);
    warn "Oops! no token" if $token eq '';
    $read.=$token;
    if ($token eq 'dnl') {
	$$ref=~s/^([^\n]*\n)//s;
	return $read.$1;
    } elsif ($token eq 'm4_pattern_forbid' or $token eq 'm4_pattern_allow' or $token eq 'pattern_forbid' or $token eq 'pattern_allow') {
	my $args=[];
	$read.=&_read_args($self,$ref,$args) if $$ref=~/^\(/;
    } elsif ($token eq 'ifdef' or $token eq 'm4_ifdef' or $token eq 'ifndef' or $token eq 'm4_ifndef') {
	# read name of macro tested, it can trigger events
	my ($unquoted, $raw)=&_read_shellword('\s+', $ref);
	$read .= $raw;

    } elsif ($KNOWN_MACRO{$token} or $token=~/^(?:AC_|AM_|CXXAC_)/) {
	my $args=[];
	$read.=&_read_args($self,$ref,$args) if $$ref=~/^\(/;
	$self->add_macro($token,$args);
    } elsif ($token eq 'if') {
	if ($$ref=~s/^(\s*(?:pkg-config|\$PKG_CONFIG)(?:\s+(--atleast-version)(?:=|\s+)(\S+))?\s+([^\s;]+)\s*)//) {
	    #if $PKG_CONFIG --atleast-version $XRANDR_REQUIRED xrandr;
	    $read .= $1;
	    my $args=[];
	    push @$args, $2, $3 if defined $3;
	    push @$args, $4;
	    if ($4 eq '--exists') {
		my ($unquoted, $raw)=&_read_shellword('\s+', $ref);
		$unquoted=~s/;$//;
		push @$args, $unquoted;
		$read .= $raw;
	    }
	    $self->add_command('pkg-config',$args);
	} else {
	    # TODO other shell tests
	}
    } elsif ($$ref=~s/^=//) {
	my ($unquoted, $raw)=&_read_shellword('[,\s]+', $ref); # (,) like in AC_FOO(BLA, z=a,z=b)
	if (defined $raw) {
	    my $val=&__kill_dnl($unquoted);
	    $self->add_var($token,$val);
	    $read .= '='.$raw;
	} else {
	    $self->add_var($token,'');
	}
    }
    return $read;
}

sub add_macro {
    my ($self,$name,$args)=@_;
    print STDERR "MACRO: $name ARGS:",join(',',@$args),"\n" if $verbose>1;
    my $hook=$self->{-macrohook};
    &$hook($name,$args,$self) if $hook;
}

sub add_command {
    my ($self,$name,$args)=@_;
    print STDERR "CMD: $name ARGS:",join(',',@$args),"\n" if $verbose>1;
    my $hook=$self->{-shellhook};
    &$hook($name,$args,$self) if $hook;
}

sub add_var {
    my ($self,$name,$val)=@_;
    print STDERR "VAR: add $name=$val\n" if $verbose>1;
    my $hook=$self->{-varhook};
    &$hook($name,$val,$self) if $hook;
    return $val;
}

sub _var_handler {
    my ($name,$val,$self)=@_;
    my $VAR=$self->{-vardata};
    $val=$1 if $val and $val=~/^\[(.*)\]$/;
    my $ref=$VAR->{$name};
    unless ($ref) {
	$ref=[];
	$VAR->{$name}=$ref;
    }
    push @$ref, $val;
}

sub _read_token {
    my ($ref)=@_;
    if ($$ref=~s/^([\w_][\w_\d]*)//s) {
	print STDERR "read token: $1\n" if $verbose>2;
	return $1;
    } else {
	return '' if $$ref eq '';
	die "Oops! read_token - no token :( ->".substr($$ref,0,1);
    }
}

sub _read_non_token_non_end_or_comments {
    my ($ref)=@_;
    my $read='';
    while (length($$ref) and $$ref !~/^[\w_\),]/) {
	if ($$ref=~s/^(\s+)//s) {
	    $read.=$1;
	} elsif ($$ref=~s/^(#[^\n]*)\n//s) {
	    print STDERR "drop comment: $1\n" if $verbose>2;
	    $read.=$1;
	} else {
	    $$ref=~s/^(.)//s;
	    $read.=$1;
	}
    }
    return $read;
}

sub _read_args {
    my ($self,$ref,$arg)=@_;
    my $read='';
    my $cur_arg='';
    if ($$ref=~s/^(\()//s) {
	$read.=$1;
    }
    while (length($$ref) and $$ref !~/^\)/) {
	if ($$ref=~s/^(\s+)//s) {
	    $read.=$1;
	    $cur_arg.=$1;
	# TODO: properly read m4 quoted string
	# TODO: load shell var assignments in m4 quoted string
	} elsif ($$ref=~s/^(\[([^]]*)\])//s) {
	    $read.=$1;
	    $cur_arg.=&__kill_dnl($2);
	} elsif ($$ref=~s/^(,)//s) {
	    $read.=$1;
	    push @$arg, $cur_arg;
	    $cur_arg='';
	} elsif ($$ref=~s/^(dnl(?:\n|\s[^\n]*\n))//s) {
	    $read.=$1;
	} else {
	    $cur_arg.=&_read_any($self,$ref);
	}
    }
    if ($$ref=~s/^(\))//s) {
	$read.=$1;
    }
    push @$arg, $cur_arg;
    return &__kill_dnl($read);
}

sub __kill_dnl {
    my $in=shift;
    unless (defined $in) {
	Carp::cluck "Oops! __kill_dnl: got undefined value";
	return;
    }
    $in=~s/^dnl\n/\n/smg;
    $in=~s/^dnl\s[^\n]*\n/\n/smg;
    $in=~s/(?<=\W)dnl\n/\n/sg;
    $in=~s/(?<=\W)dnl\s[^\n]*\n/\n/sg;
    return $in;
}

# regexp is copypasted from Text/ParseWords.pm
sub _read_shellword {
    my($delimiter, $lineref) = @_;
    my($word);

    no warnings 'uninitialized';	# we will be testing undef strings

    return '' if not length($$lineref);
        # This pattern is optimised to be stack conservative on older perls.
        # Do not refactor without being careful and testing it on very long strings.
        # See Perl bug #42980 for an example of a stack busting input.
    $$lineref =~ s/^
                    (?: 
                        # double quoted string
                        (")                             # $quote
                        ((?>[^\\"]*(?:\\.[^\\"]*)*))"   # $quoted 
		    |	# --OR--
                        # singe quoted string
                        (')                             # $quote
                        ((?>[^\\']*(?:\\.[^\\']*)*))'   # $quoted
# by viy@; my patch here -----------------------------------------------
		    |	# --OR--
                        # backtick quoted string
                        (`)                             # $quote
                        ((?>[^\\`]*(?:\\.[^\\`]*)*))`   # $quoted
# by viy@; end my patch here -----------------------------------------------
                    |   # --OR--
                        # unquoted string
		        (                               # $unquoted 
                            (?:\\.|[^\\"'])*?           
                        )		
                        # followed by
		        (                               # $delim
                            \Z(?!\n)                    # EOL
                        |   # --OR--
                            (?-x:$delimiter)            # delimiter
                        |   # --OR--                    
                            (?!^)(?=["'])               # a quote
                        )  
		    )//xs or return;		# extended layout                  
       # viy@ patch broke numeration
       #my ($quote, $quoted, $unquoted, $delim) = (($1 ? ($1,$2) : ($3,$4)), $5, $6);
    my ($unquoted, $delim) = ($7, $8);
    my ($quote, $quoted);
    if ($1) {
	($quote, $quoted) = ($1,$2);
    } elsif ($3) {
	($quote, $quoted) = ($3,$4);
    } elsif ($5) {
	($quote, $quoted) = ($5,$6);
    }

    return '' unless( defined($quote) || length($unquoted) || length($delim));

    my $raw = "$quote$quoted$quote$delim";

    $unquoted =~ s/\\(.)/$1/sg;
    if (defined $quote) {
	$quoted =~ s/\\(.)/$1/sg if ($quote eq '"');
	$quoted =~ s/\\([\\'])/$1/g if ( $PERL_SINGLE_QUOTE && $quote eq "'");
    }
    my $processed = defined $quote ? $quoted : $unquoted;
    return ($processed, $raw);
}




1;
