package RPM::Source::Dependency::Analyzer::Collector::Plugin::CMake;

use strict;
use warnings;

use Carp;
use Marpa::R2;
use Data::Dumper;
use RPM::Source::Dependency::Analyzer::Collector::PkgDB;
use RPM::Source::Dependency::Analyzer::Collector::Parser::CMake::Marpa;

our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser/;

#0-silent
#1-parse errors
#2-add lib/path/pkgcofig add report
#3-args report
#4-token report


# TODO: alternatives:
#GLUT -> pkg-config: glut|freeflut

our $verbose=0;
our $VERSION=0.004;

sub parse_string {
    my ($self,$input,$inputname)=@_;

    my $g = $RPM::Source::Dependency::Analyzer::Collector::Parser::CMake::Marpa::grammar;

    my $re = Marpa::R2::Scanless::R->new(
	{ grammar => $g,
	  semantics_package => 'RPM::Source::Dependency::Analyzer::Collector::Parser::CMake::Marpa',
	  #Marpa: Scanless G1 Earley item count (252) exceeds warning threshold
	  too_many_earley_items => 1024,
	});

    eval {
	$re->read(\$input);
    };
    if ($@) {
	if (defined $inputname) {
	    die "FATAL: CMAKE: while parsing $inputname: $@";
	} else {
	    Carp::croak "FATAL: CMAKE: parse_string:\n$input\n:GOT:\n$@";
	}
    }

    my $value = $re->value;
    unless (defined $value) {
	if (defined $inputname) {
	    warn "FATAL: CMAKE: can't parse $inputname";
	} else {
	    Carp::carp "FATAL: CMAKE: can't parse string:\n$input";
	}
	return;
    }
    $value = ${$value};

    foreach my $token (@$value) {
	if (! ref($token)) { # eq 'ARRAY'
	    print STDERR "CMAKE:TOKEN: $token\n" if $verbose>3;
	    print STDERR "CMAKE: uninspected nonref: $token\n" if $verbose;
	} elsif (! ref($token) eq 'ARRAY') { # eq 'ARRAY'
	    print STDERR "CMAKE:TOKEN: $token\n" if $verbose>3;
	    print STDERR "CMAKE: uninspected non-ARRAY: $token\n" if $verbose;
	} elsif ($token->[0] =~ '^\s+$' or $token->[0] =~ '^#') {
	    # space, comments ...
	} elsif ($token->[0] eq '(') {
	    print STDERR "CMAKE: uninspected loose parenthesis:",Dumper($token),"\n" if $verbose;
	} elsif ($token->[1] eq '(' and $token->[2] eq ')') {
	    &_process_cmake_op_noarg($self,lc($token->[0]));
	    print STDERR "CMAKE:TOKEN: CMD $token->[0] ()\n" if $verbose>3;
	} elsif ($token->[1] eq '(' and $token->[3] eq ')') {
	    &_process_cmake_op($self,lc($token->[0]),$token->[2]);
	    print STDERR "CMAKE:TOKEN: CMD $token->[0] (",Dumper($token->[2]),")\n" if $verbose>3;
	} else {
	    print STDERR 'CMAKE: ',
	    $inputname ? "while parsing $inputname: ": "parse_string:\n$input\n:GOT: ",
	    "uninspected something:",Dumper($token),"\n" if $verbose;
	}
    }

    #my $hook=$self->{-postfilehook};
    #&$hook($self) if $hook;
}

sub finish {
    my ($self)=@_;
    #The top-level CMakeLists.txt file for a project must contain a literal, direct call to the project() command; loading one through the include() command is not sufficient. If no such call exists CMake will implicitly add one to the top that enables the default languages (C and CXX).
    unless ($self->{project_found}) {
	$self->add_flag('c');
	$self->add_flag('cxx');
    }
}

# TODO move to main resolver.pm
my %CMAKE_DROP=map{$_=>1} qw/
UnixCommands
__system_type___linux.etc___
System
___in_glibc_devel___
Iconv
Libintl
LibIntl
intl
Intl
__mac___
Carbon
__win___
DbgHelp
DirectX
DirectXSDK
D3DX11Effects
GNUWIN
GnuWin32
VLD
/;

# TODO: ?
#wxWidgets -> wxGTK-devel?

## Here you can define, what Libraries of wxWidgets you need for your Application. You can figure out what Libraries you need here:
## http://www.wxwidgets.org/manuals/2.8/wx_librarieslist.html
## order matters!
#SET(wxWidgets_USE_LIBS net aui xml qa richtext html adv core base)
## We need the Find package for wxWidgets to work, but auto-find is broken in x-compile setup
#FIND_PACKAGE(wxWidgets REQUIRED ${wxWidgets_USE_LIBS})

my %CMAKE2HEADERS=qw!
XKB X11/XKBlib.h
PCIUTILS	pci/pci.h
!;

my %CMAKE2LIB=qw/
ACL	acl
Alure	alure
ALSA	asound
APOGEE	apogeee;apogeeu
Armadillo	armadillo
ASPELL	aspell
Augeas	augeas
Bliss	bliss
Boost::atomic	boost_atomic
Boost::chrono	boost_chrono
Boost::container	boost_container
Boost::context	boost_context
Boost::coroutine	boost_coroutine
Boost::date_time	boost_date_time
Boost::filesystem	boost_filesystem
Boost::filesystem-st	boost_filesystem-st
Boost::graph	boost_graph
Boost::graph_parallel	boost_graph_parallel
Boost::iostreams	boost_iostreams
Boost::locale	boost_locale
Boost::log	boost_log
Boost::log_setup	boost_log_setup
Boost::math_c99f	boost_math_c99f
Boost::math_c99l	boost_math_c99l
Boost::math_c99	boost_math_c99
Boost::math_tr1f	boost_math_tr1f
Boost::math_tr1l	boost_math_tr1l
Boost::math_tr1	boost_math_tr1
Boost::mpi	boost_mpi
Boost::mpi_python	boost_mpi_python
Boost::prg_exec_monitor	boost_prg_exec_monitor
Boost::program_options	boost_program_options
Boost::python3	boost_python3
Boost::python	boost_python
Boost::random	boost_random
Boost::regex	boost_regex
Boost::serialization	boost_serialization
Boost::signals	boost_signals
Boost::system	boost_system
Boost::thread	boost_thread
Boost::timer	boost_timer
Boost::unit_test_framework	boost_unit_test_framework
Boost::wave	boost_wave
Boost::wserialization	boost_wserialization
BZip2	bz2
Cairo	cairo
Canberra	canberra
Cauchy	m2mml
CFITSIO	cfitsio
CGAL	CGAL
CGAL::Core	CGAL_Core
CGAL::ImageIO	CGAL_ImageIO
CGAL::Qt3	CGAL_Qt3
CGAL::Qt4	CGAL_Qt4
CGSI_GSOAP	cgsi_plugin
CppUnit	cppunit
CUPS	cups
CURL	curl
Curses	curses
DBI	dbi
Discount	markdown
DNET	dnet
EGL	EGL
Elektra	elektra
Event	event
LibEpoxy	epoxy
epoxy	epoxy
EPub	epub
Exiv2	exiv2
Falcon	falcon_engine
FAM	fam
Fltk	fltk
Fontconfig	fontconfig
Freetype	freetype
gbm	gbm
Gcrypt	gcrypt
GD	gd
GIF	gif;ungif
GIO	gio-2.0
GLESv1	GLESv1
GLESv2	GLESv2
GLEW	GLEW
GLIB	glib-2.0
GLIB2	glib-2.0
Globus_COMMON	globus_common
Globus_GSI_CERT_UTILS	globus_gsi_cert_utils
Globus_GSI_CREDENTIAL	globus_gsi_credential
Globus_GSSAPI_GSI	globus_gssapi_gsi
Globus_GSS_ASSIST	globus_gss_assist
GMP	gmp
GnuTLS	gnutls
Gpgme	gpgme
GSL	gsl
gSOAP	gsoap
GTest	gtest
GTK2	gdk-x11-2.0;gtk-x11-2.0
GTK3	gdk-3;gtk-3
HSPELL	hspell
HUNSPELL	hunspell
ICU::uc	icuuc
ICU::dt	icutu
ICU::data	icudata
ICU::in	icuio
ICU::i18n	icui18n
INDI	indi
Iniparser	iniparser
Jack	jack
JACK	jack
JPEG	jpeg
KF5Prison	prison
LASH	lash
LCMS2	lcms2
LCMS	lcms
LibAV::AVCODEC  avcodec
LibAV::AVFORMAT avformat
LibAV::AVDEVICE avdevice
LibAV::AVFILTER avfilter
LibAV::AVRESAMPLE avresample
LibAV::AVUTIL   avutil
LibAV::SWSCALE  swscale
Libcap	cap
Libdrm	drm
LibFFTW3	fftw3
FFTW3	fftw3
LibGcrypt	gcrypt
LibGit2	git2
libgps	gps
LibHaru	haru
libhybris	hybris
Libhybris	hybris
LibHybris	hybris
LibIDN	idn
Libinput	input
LibLZMA	lzma
Libpinyin	pinyin
LibPNG	png
LibRaw	raw_r
LibRTMP	rtmp
LibSamplerate	samplerate
LibXslt	xslt
LMDB	lmdb
Log4C	log4c
LZO	lzo2
Mikmod	mikmod
MiniZip minizip
Minizip minizip
MNG	mng
Mtp	mtp
NET	net
Ogg	ogg
OGG	ogg
OpenAL  openal
OpenConnect	openconnect
OpenGL	GL;GLU
OpenGLES	GLESv2
OSS	oss
PCAP	pcap
PCIUTILS	pci;resolv
PCRE	pcre
PhysFS  physfs
PNG     png
PopplerQt4	poppler-qt4
PopplerQt5	poppler-qt5
Portaudio	portaudio
Protobuf	protobuf
Pthread	pthread
PWQuality	pwquality

Qt4::QtAssistantClient	QtAssistantClient
Qt4::Qt3Support	Qt3Support
Qt4::QtCLucene	QtCLucene
Qt4::QtCore	QtCore
Qt4::QtDBus	QtDBus
Qt4::QtDeclarative	QtDeclarative
Qt4::QtDesigner	QtDesigner
Qt4::QtDesignerComponents	QtDesignerComponents
Qt4::QtGui	QtGui
Qt4::QtHelp	QtHelp
Qt4::QtMultimedia	QtMultimedia
Qt4::QtNetwork	QtNetwork
Qt4::QtOpenGL	QtOpenGL
Qt4::QtScript	QtScript
Qt4::QtScriptTools	QtScriptTools
Qt4::QtSql	QtSql
Qt4::QtSvg	QtSvg
Qt4::QtTest	QtTest
Qt4::QtUiTools	QtUiTools
Qt4::QtWebKit	QtWebKit
Qt4::QtXml	QtXml
Qt4::QtXmlPatterns	QtXmlPatterns

Qt5::Concurrent	Qt5Concurrent
Qt5::Core	Qt5Core
Qt5::DBus	Qt5DBus
Qt5::EglDeviceIntegration	Qt5EglDeviceIntegration
Qt5::Gui	Qt5Gui
Qt5::Network	Qt5Network
Qt5::OpenGL	Qt5OpenGL
Qt5::PrintSupport	Qt5PrintSupport
Qt5::Sql	Qt5Sql
Qt5::Test	Qt5Test
Qt5::Widgets	Qt5Widgets
Qt5::XcbQpa	Qt5XcbQpa
Qt5::Xml	Qt5Xml
Qt5::Bluetooth	Qt5Bluetooth
Qt5::Nfc	Qt5Nfc
Qt5::Qml	Qt5Qml
Qt5::Quick	Qt5Quick
Qt5::QuickParticles	Qt5QuickParticles
Qt5::QuickTest	Qt5QuickTest
Qt5::QuickWidgets	Qt5QuickWidgets
Qt5::GLib-2.0	Qt5GLib-2.0
Qt5::GStreamer-1.0	Qt5GStreamer-1.0
Qt5::GStreamerQuick-1.0	Qt5GStreamerQuick-1.0
Qt5::GStreamerUi-1.0	Qt5GStreamerUi-1.0
Qt5::GStreamerUtils-1.0	Qt5GStreamerUtils-1.0
Qt5::Location	Qt5Location
Qt5::Positioning	Qt5Positioning
Qt5::Multimedia	Qt5Multimedia
Qt5::MultimediaQuick_p	Qt5MultimediaQuick_p
Qt5::MultimediaWidgets	Qt5MultimediaWidgets
Qt5::Declarative	Qt5Declarative
Qt5::Script	Qt5Script
Qt5::ScriptTools	Qt5ScriptTools
Qt5::Sensors	Qt5Sensors
Qt5::SerialPort	Qt5SerialPort
Qt5::TextToSpeech	Qt5TextToSpeech
Qt5::Svg	Qt5Svg
Qt5::CLucene	Qt5CLucene
Qt5::Designer	Qt5Designer
Qt5::DesignerComponents	Qt5DesignerComponents
Qt5::Help	Qt5Help
Qt5::Compositor	Qt5Compositor
Qt5::WaylandClient	Qt5WaylandClient
Qt5::WebChannel	Qt5WebChannel
Qt5::WebKit	Qt5WebKit
Qt5::WebKitWidgets	Qt5WebKitWidgets
Qt5::WebSockets	Qt5WebSockets
Qt5::X11Extras	Qt5X11Extras
Qt5::XmlPatterns	Qt5XmlPatterns

Qwt	qwt
RAW1394	raw1394
ReadLine	readline
Readline	readline
SCTP	sctp
SDL     SDL
SDL_gles	SDL_gles
SDL_image	SDL_image
SDL_mixer	SDL_mixer
SDL_net	SDL_net
SDL_sound	SDL_sound
SDL_ttf	SDL_ttf
SDL2     SDL2
SDL2_gles	SDL2_gles
SDL2_image	SDL2_image
SDL2_mixer	SDL2_mixer
SDL2_net	SDL2_net
SDL2_sound	SDL2_sound
SDL2_ttf	SDL2_ttf
Sensors	sensors
SFML::Audio	sfml-audio
SFML::Graphics	sfml-graphics
SFML::Network	sfml-network
SFML::System	sfml-system
SFML::Window	sfml-window
SQLite3	sqlite3
Sqlite3	sqlite3
SQLite	sqlite3
Sqlite	sqlite3
Taglib	tag
Theora	theora
Threads pthread
Thrift	thrift
TIFF	tiff
Tolua++	tolua++
UDev	udev
Upnp	upnp
USB	usb
UTEMPTER	utempter
VOIKKO	voikko
Vorbis	vorbis
VorbisFile	vorbisfile
Wayland::Client	wayland-client
Wayland::Cursor	wayland-cursor
Wayland::Egl	wayland-egl
Wayland::Server	wayland-server
X11     X11
X11_XCB     X11-xcb
XCB::XCB	xcb
XCB::RENDER	xcb-render
XCB::SHAPE	xcb-shape
XCB::XFIXES	xcb-xfixes
XCB::SHM	xcb-shm
XCB::ATOM	xcb-util
XCB::AUX	xcb-util
XCB::COMPOSITE	xcb-composite
XCB::CURSOR	xcb-cursor
XCB::DAMAGE	xcb-damage
XCB::DPMS	xcb-dpms
XCB::DRI2	xcb-dri2
XCB::DRI3	xcb-dri3
XCB::EVENT	xcb-util
XCB::EWMH	xcb-ewmh
XCB::GLX	xcb-glx
XCB::ICCCM	xcb-icccm
XCB::IMAGE	xcb-image
XCB::KEYSYMS	xcb-keysyms
XCB::PRESENT	xcb-present
XCB::RANDR	xcb-randr
XCB::RECORD	xcb-record
XCB::RENDERUTIL	xcb-renderutil
XCB::RES	xcb-res
XCB::SCREENSAVER	xcb-screensaver
XCB::SYNC	xcb-sync
XCB::UTIL	xcb-util
XCB::XEVIE	xcb-xevie
XCB::XF86DRI	xcb-xf86dri
XCB::XINERAMA	xcb-xinerama
XCB::XINPUT	xcb-xinput
XCB::XKB	xcb-xkb
XCB::XPRINT	xcb-xprint
XCB::XTEST	xcb-xtest
XCB::XV	xcb-xv
XCB::XVMC	xcb-xvmc
Xcm	Xcm
Xfixes	Xfixes
Xinerama	Xinerama
XKBCommon	xkbcommon
XRandR	Xrandr
Xxf86vm	Xxf86vm
Yajl	yajl
ZLIB    z
Zlib    z
/;
$CMAKE2LIB{'XCB'}=[qw/
xcb
xcb-composite
xcb-cursor
xcb-damage
xcb-dpms
xcb-dri2
xcb-dri3
xcb-ewmh
xcb-glx
xcb-icccm
xcb-image
xcb-keysyms
xcb-present
xcb-randr
xcb-record
xcb-render
xcb-res
xcb-screensaver
xcb-shape
xcb-shm
xcb-sync
xcb-util
xcb-xevie
xcb-xf86dri
xcb-xfixes
xcb-xinerama
xcb-xkb
xcb-xtest
xcb-xv
xcb-xvmc
/,
# not in Sisyphus (in LIB_IGNORE)
'xcb-renderutil',
'xcb-xprint',
];
$CMAKE2LIB{'LibAV'}=[qw/avcodec avformat avutil/];
$CMAKE2LIB{'ICU'}=[qw/icuuc icutu icudata icui18n/];
$CMAKE2LIB{'GTK2::gtk'}=[qw/
        glib-2.0
        atk-1.0
        cairo
        gdk-pixbuf-2.0
    gdk-x11-2.0
    gtk-x11-2.0
        pango-1.0
        sigc++-2.0
/];
$CMAKE2LIB{'GTK2::gtkmm'}=[qw/
        glibmm-2.4
        glib-2.0
        atk-1.0
        atkmm-1.6
        cairo
        cairomm-1.0
        gdk-pixbuf-2.0
        gdkmm-2.4
        giomm-2.4
    gdk-x11-2.0
    gtk-x11-2.0
        gtkmm-2.4
        pango-1.0
        pangomm-1.4
        sigc++-2.0
/];
$CMAKE2LIB{'GTK2::glade'}=[qw/
        glib-2.0
        atk-1.0
        cairo
        gdk-pixbuf-2.0
    gdk-x11-2.0
    gtk-x11-2.0
        libglade-2.0
        pango-1.0
/];
$CMAKE2LIB{'GTK2::glademm'}=[qw/
        glibmm-2.4
        glib-2.0
        atk-1.0
        atkmm-1.6
        cairo
        cairomm-1.0
        gdk-pixbuf-2.0
        gdkmm-2.4
        giomm-2.4
    gdk-x11-2.0
    gtk-x11-2.0
        gtkmm-2.4
        libglade-2.0
        libglademm-2.4
        pango-1.0
        pangomm-1.4
        sigc++-2.0
/];

# support components
#FIND_PACKAGE(Boost 1.35.0 COMPONENTS unit_test_framework)
#FIND_PACKAGE(Boost 1.42.0 COMPONENTS thread system REQUIRED)
#FIND_PACKAGE(Boost 1.42.0 REQUIRED)
$CMAKE2LIB{'Boost'}=[# TODO share pkg-independent with autoconf
qw/
 boost_atomic
 boost_chrono
 boost_container
 boost_date_time
 boost_graph
 boost_iostreams
 boost_prg_exec_monitor
 boost_random
 boost_regex
 boost_serialization
 boost_system
 boost_thread
 boost_timer
 boost_unit_test_framework
 boost_wserialization
/];
# boost_context
# boost_coroutine
# boost_filesystem-st
# boost_filesystem
# boost_graph_parallel
# boost_locale
# boost_log
# boost_log_setup
# boost_math_c99
# boost_math_c99f
# boost_math_c99l
# boost_math_tr1
# boost_math_tr1f
# boost_math_tr1l
# boost_mpi
# boost_mpi_python
# boost_program_options
# boost_python3
# boost_python
# boost_signals
# boost_wave

my %CMAKE2PATH=(qw/
ASPELL	aspell
Automoc4	automoc4
BISON	bison
CatDoc	catdoc
Doxygen		doxygen
FLEX	flex
Gettext	gettext
GetText	gettext
Git	git
GSSAPI	krb5-config
HSPELL	hspell
HUNSPELL	hunspell
IBus	ibus
IntlTool	intltoolize
LATEX	latex
MakeNSIS	makensis
Msgfmt	msgfmt
MySQL	mysql
OpenConnect	openconnect
PkgConfig	pkg-config
Subversion	svn
SWIG	swig
Vala	vala
VOMS	voms
WaylandScanner	wayland-scanner
/, #find_package(ImageMagick COMPONENTS convert)
qw/
ImageMagick::convert	convert
/);

# no need: /usr/lib/cmake/Qt5LinguistTools => qt5-tools-devel req qt5-tools
#$CMAKE2PATH{'Qt5LinguistTools'}=[qw/lconvert-qt5 lrelease-qt5 lupdate-qt5/];

#TODO: in %_libdir/hdf5-seq/lib/pkgconfig
#NETCDF	netcdf

#MediaPlayerInfo=/usr/share/media-player-info/sony_psp.mpi
my %CMAKE2RPM=qw/
DocBookXML4 docbook-dtds
DocBookXSL  docbook-style-xsl
ECM extra-cmake-modules
MediaPlayerInfo	media-player-info
Java::Development java-devel-default
Java::Runtime java-devel-default
/;

#kde4plasma-addons-devel
#Lancelot
#Lancelot-Datamodels
#kde4plasma-addons-devel

my %CMAKE2PKGCNF=qw/
ALUT freealut
Atk atk
Avahi avahi-core
CEGUI CEGUI-0
Irrlicht CEGUI-0-IRRLICHT
Check check
Clutter clutter-1.0
DBus dbus-1
DNSSD avahi-core
Edit libedit
Enet libenet
Enchant enchant
Evdev	xorg-evdev
FFI	libffi
FFmpeg::AVCODEC  libavcodec
FFmpeg::AVFORMAT libavformat
FFmpeg::AVDEVICE libavdevice
FFmpeg::AVUTIL   libavutil
FFmpeg::SWSCALE  libswscale
FFmpeg::SWRESAMPLE libswresample
FFmpeg::POSTPROC libpostproc
FFTW3f fftw3f
__a_choice_of_GLUT glut_or_freeglut
GObject gobject-2.0
GObjectIntrospection gobject-introspection-1.0
GTK3::gtksourceview gtksourceview-3.0
GTK3::gobject gobject-2.0
GTK3::gio gio-2.0
GTK3::gthread gthread-2.0
GTK3::glib glib-2.0                 
IBus ibus-1.0
ImageMagick	MagickCore
ImageMagick::Magick++	Magick++
ImageMagick::MagickCore	MagickCore
ImageMagick::MagickWand	MagickWand
LibMemcached libmemcached
Libsmbclient smbclient
LibSmbClient smbclient
LibSMBclient smbclient
LibSndFile sndfile
LIBUNSHIELD libunshield
LibXml2 libxml-2.0
Loudmouth loudmouth-1.0
Lua lua
MsgPack msgpack
MobileBroadbandProviderInfo mobile-broadband-provider-info
ModemManager ModemManager
ModPlug libmodplug
Neon neon
ODE ode
OGRE OGRE
Ogre OGRE
OIS OIS
OpenCC opencc
OpenCV opencv
OpenEXR OpenEXR
OpenGLES2 glesv2
__TODO__choice_over_libre_OpenSSL	openssl
Pango pango
PopplerQt4 poppler-qt4
PostgreSQL libpq
Qalculate libqalculate
QDBM qdbm
QHttpEngine qhttpengine
qhttpengine qhttpengine
SampleRate samplerate
SCIM scim
SharedMimeInfo shared-mime-info
Speex speex
Synaptics xorg-synaptics
TagLib taglib
TinyXML tinyxml2
TokyoCabinet tokyocabinet
XKBCommon::XKBCommon xkbcommon
XkbFile xkbfile
UUID uuid
USB-1 libusb-1.0
USB1 libusb-1.0
ZeroMQ libzmq

__TODO__ __MIGHT_CHANGE__
ORC orc-0.4
/;
$CMAKE2PKGCNF{'OggVorbis'}=[qw/ogg oggz vorbis vorbisenc vorbisfile/];
$CMAKE2PKGCNF{'CEGUIOGRE'}=[qw/CEGUI-0 OGRE/];
$CMAKE2PKGCNF{'NetworkManager'}=[qw/NetworkManager libnm-util/];
$CMAKE2PKGCNF{'FFmpeg'}=[qw/libavcodec libavformat libavdevice libavutil libswscale libpostproc/];
$CMAKE2PKGCNF{'FFMPEG'}=$CMAKE2PKGCNF{'FFmpeg'};
$CMAKE2PKGCNF{'GLFW'}=[qw/libglfw glfw3/];
#$CMAKE2PKGCNF{''}=[qw//];


my %CMAKE2ENV=qw/
Java	java
JNI	java
KDE4	kde4
KF5	kf5
OpenMP	openmp
Perl	perl
PerlLibs	perl
PerlMore	perl
Python2Interp	python2
Python2Libs	python2
Python2LinkLibs	python2
PythonInterp	python
PythonLibs	python
PythonLinkLibs	python
Ruby	ruby
Qt4	qt4
wxWidgets wxWidgets
/;
#Qt5	qt5

# todo: pkg?

# todo: headers?
#Libintl	libintl.h glibc-devel?


my %_pkgconfig_opt=map {$_=>1} qw/REQUIRED QUIET NO_CMAKE_PATH NO_CMAKE_ENVIRONMENT_PATH/;
my %_findpost_opt=map {$_=>1} qw/REQUIRED NAMES COMPONENTS HINTS EXACT QUIET
NO_POLICY_SCOPE HINTS CONFIG NO_MODULE OPTIONAL_COMPONENTS
CONFIGS HINTS PATHS PATH_SUFFIXES DOC NO_DEFAULT_PATH 
 NO_CMAKE_ENVIRONMENT_PATH NO_CMAKE_PATH NO_SYSTEM_ENVIRONMENT_PATH 
 NO_CMAKE_PACKAGE_REGISTRY NO_CMAKE_BUILDS_PATH NO_CMAKE_SYSTEM_PATH
 NO_CMAKE_SYSTEM_PACKAGE_REGISTRY
 CMAKE_FIND_ROOT_PATH_BOTH ONLY_CMAKE_FIND_ROOT_PATH NO_CMAKE_FIND_ROOT_PATH/;

# cmd is forcefully lowercased; args are not
sub _process_cmake_op {
    my ($self,$cmd,$args)=@_;
    print STDERR 'CMAKE:PROCESS: CMD=',$cmd,' (',Dumper($args),")\n" if $verbose>2;

    if ($cmd eq 'project') {
	my $proj=shift @$args;
	print STDERR 'DEBUG: CMAKE: in project1: project args=(',Dumper($args),")\n" if $verbose>3;
	shift @$args while scalar @$args and uc($args->[0]) ne 'LANGUAGES';
	print STDERR 'DEBUG: CMAKE: in project2: LANGUAGES args=(',Dumper($args),")\n" if $verbose>2;
	my @langs;
	if (scalar @$args) {
	    shift @$args if @$args>0 and uc($args->[0]) eq 'LANGUAGES';
	    push @langs, shift @$args while scalar @$args and uc($args->[0]) ne 'NONE' and uc($args->[0]) ne 'VERSION';
	    print STDERR 'DEBUG: CMAKE: in project3: LANGUAGES args=(',Dumper($args),")\n" if $verbose>3;
	    foreach my $lang (@langs) {
		if (lc($lang) eq 'c') {
		    $self->add_flag('c');
		} elsif (lc($lang) eq 'cxx') {
		    $self->add_flag('cxx');
		} elsif (lc($lang) eq 'fortran') {
		    $self->add_flag('fortran');
		} else {
		    warn "CMAKE:TODO: unsupported language=$lang";
		}
	    }
	}
	unless (@langs) {
	    # default cmake languages
	    $self->add_flag('c');
	    $self->add_flag('cxx');
	}
	$self->{project_found}=1;

    #macro(FINDLIB lib)find_library(${ARGV})mark_as_advanced(${lib})endmacro(FINDLIB)
    } elsif ($cmd eq 'find_library' or $cmd eq 'FINDLIB') {
	#find_library(SDL2_LIBRARY_TEMP NAMES SDL2 HINTS ENV SDL2DIR PATH_SUFFIXES lib)
	my $var=shift @$args;
	my $opt=shift @$args;
	if (! defined($opt)) {
	    print STDERR "CMAKE:find_library($var): strange :(\n" if $verbose and $var!~/^\$/;
	} elsif (uc($opt) ne 'NAMES') {
	    $self->add_lib($opt);
	} else {
	    my @names;
	    while (scalar @$args and not $_findpost_opt{uc($args->[0])}) {
		push @names, shift @$args;
	    }
	    $self->add_lib(@names<2 ? @names:[@names]);
	}

    } elsif ($cmd eq 'find_program') {
	#find_program(FLEX_EXECUTABLE NAMES flex PATHS /usr/bin DOC "Flex executable")
	my $var=shift @$args;
	my $opt=shift @$args;
	if (uc($opt) ne 'NAMES') {
	    $self->add_path($opt);
	} else {
	    my @names;
	    while (scalar @$args and not $_findpost_opt{uc($args->[0])}) {
		push @names, shift @$args;
	    }
	    $self->add_path(@names<2 ? @names:[@names]);
	}

    } elsif ($cmd eq 'find_package') {
	#find_package(Boost REQUIRED)

	#find_package(<package> [version] EXACT QUIET
        #     REQUIRED] COMPONENTS
        #     CONFIG NO_MODULE
        #     NO_POLICY_SCOPE

	# first arg is pkgname -- required
	my $pkg=shift @$args;
	my $components_found;
	while (scalar @$args) {
	    my $word=shift @$args;
	    if (uc($word) eq 'NAMES') {
		my @names;
		while (scalar @$args and not $_findpost_opt{uc($args->[0])}) {
		    push @names, shift @$args;
		}
		$self->add_cmakepkg(@names<2 ? @names:[@names]);
	    } elsif (uc($word) eq 'COMPONENTS' or uc($word) eq 'OPTIONAL_COMPONENTS') {
		$components_found=1;
		my @components;
		while (scalar @$args and not $_findpost_opt{uc($args->[0])}) {
		    push @components, $pkg.'::'.shift(@$args);
		}
		$self->add_cmakepkg(@components);
	    }
	}
	$self->add_cmakepkg($pkg) unless $components_found;

    } elsif ($cmd eq 'pkg_search_module' or
	     $cmd eq 'pkg_check_modules'
	) {
	my $var=shift @$args;
	while (scalar @$args and $_pkgconfig_opt{$args->[0]}) {
	    shift @$args;
	}
	$self->add_pkgconfig(@$args);
	

    } elsif ($cmd eq 'include') {
	#INCLUDE(FindPkgConfig)
	my $file=shift @$args;
	if ($file=~s/^(?:Find|Use)//) {
	    $self->add_cmakepkg($file);	    
	}

    } elsif ($cmd eq 'set') {
	my $var=uc(shift @$args);
	if ($var eq 'CMAKE_CXX_FLAGS') {
	    $self->add_flag('cxx');
	}

#TODO

#PKG_SEARCH_MODULE(SDL2 REQUIRED sdl2)
# careful as local libraries are included
#target_link_libraries
# example: (my old patch for springlobby)
#add_library(CurlWrapper Downloader/CurlWrapper.cpp)
#target_link_libraries(CurlWrapper curl)


    }
}

# cmd is forcefully lowercased
sub _process_cmake_op_noarg {
    my ($self,$cmd)=@_;
}

sub add_cmakepkg {
    my $self=shift;
    foreach my $cmakepkg (@_) {
	&__add_cmakepkg_set($self,$cmakepkg);
    }
}

sub __add_cmakepkg_set {
    my ($self,$cmakepkgset)=@_;
    if (! ref($cmakepkgset)) {
	&__add_one_cmakepkg($self,$cmakepkgset);
    } else {
	my $found=0;
	foreach my $cmakepkg (@$cmakepkgset) {
	    $found=1 if &__resolve_cmakepkg($self,$cmakepkg);
	    last if $found;
	}
	# todo: check whether it works - add them as 
	# $storage->add_cmake_req($cmakepkgset);
	warn "CMAKE: TODO: alternative search for ",join(":",@$cmakepkgset), "found nothing" unless $found;
    }
}

sub __add_one_cmakepkg {
    my ($self,$cmakepkg)=@_;
    my $found=&__resolve_cmakepkg($self,$cmakepkg);
    if (!$found) {
	my $storage=$self->{-storage};
	$storage->add_cmake_req($cmakepkg);
	print STDERR "info: CMAKE: find_package($cmakepkg) passed to resolver\n" if $verbose;
    }
}

sub __resolve_cmakepkg {
    my ($self,$cmakepkg)=@_;
    my $found=0;
    $cmakepkg=~s/\.cmake$//;
    return $found if $CMAKE_DROP{$cmakepkg};
    my $lib=$CMAKE2LIB{$cmakepkg};
    if ($lib) {
	$self->add_lib(ref ($lib) ? @$lib: split(';',$lib));
	$found=1;
    }
    my $hdrs=$CMAKE2HEADERS{$cmakepkg};
    if ($hdrs) {
	$self->add_header(ref ($hdrs) ? @$hdrs: split(';',$hdrs));
	$found=1;
    }
    my $path=$CMAKE2PATH{$cmakepkg};
    if ($path) {
	$self->add_path(ref ($path) ? @$path: split(';',$path));
	$found=1;
    }
    my $pkgcnf=$CMAKE2PKGCNF{$cmakepkg};
    if ($pkgcnf) {
	$self->add_pkgconfig(ref ($pkgcnf) ? @$pkgcnf: split(';',$pkgcnf));
	$found=1;
    }
    my $rpmname=$CMAKE2RPM{$cmakepkg};
    if ($rpmname) {
	$self->add_pkg(ref ($rpmname) ? @$rpmname: split(';',$rpmname));
	$found=1;
    }
    my $pkgenv=$CMAKE2ENV{$cmakepkg};
    if ($pkgenv) {
	my $storage=$self->{-storage};
	&RPM::Source::Dependency::Analyzer::Collector::PkgDB::db_add_environment($storage,ref ($pkgenv) ? @$pkgenv: split(';',$pkgenv));
	$found=1;
    }
    return $found;
}

sub add_flag {
    my $self=shift;
    print STDERR 'CMAKE:FLAG: add ',join(' ',@_),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    map {$storage->set_lang_flag($_)} @_;
}

sub add_lib {
    my $self=shift;
    print STDERR 'CMAKE:LIB: add ',join(' ',@_),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    $storage->add_libs_req(@_);
}

sub add_header {
    my $self=shift;
    print STDERR 'CMAKE:HEADER: add ',join(' ',@_),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    $storage->add_hdrs_req(@_);
}

sub add_path {
    my $self=shift;
    print STDERR 'CMAKE:PATH: add ',join(' ',@_),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    $storage->add_path_req(@_);
}

sub add_pkg {
    my $self=shift;
    print STDERR 'CMAKE:PKG: add ',join(' ',@_),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    $storage->add_pkgs_req(@_);
}

sub add_pkgconfig {
    my $self=shift;
    my @names=map {s,[<=>]+.+$,, ; $_} @_;
    print STDERR 'CMAKE:PKGCNF: add ',join(' ',@names),"\n" if $verbose>1;
    my $storage=$self->{-storage};
    $storage->add_pkgconf_req(@names);
}



1;

__END__

Qt5Concurrent	Qt5Concurrent
Qt5Core	Qt5Core
Qt5DBus	Qt5DBus
Qt5EglDeviceIntegration	Qt5EglDeviceIntegration
Qt5Gui	Qt5Gui
Qt5Network	Qt5Network
Qt5OpenGL	Qt5OpenGL
Qt5PrintSupport	Qt5PrintSupport
Qt5Sql	Qt5Sql
Qt5Test	Qt5Test
Qt5Widgets	Qt5Widgets
Qt5XcbQpa	Qt5XcbQpa
Qt5Xml	Qt5Xml
Qt5Bluetooth	Qt5Bluetooth
Qt5Nfc	Qt5Nfc
Qt5Qml	Qt5Qml
Qt5Quick	Qt5Quick
Qt5QuickParticles	Qt5QuickParticles
Qt5QuickTest	Qt5QuickTest
Qt5QuickWidgets	Qt5QuickWidgets
Qt5GLib-2.0	Qt5GLib-2.0
Qt5GStreamer-1.0	Qt5GStreamer-1.0
Qt5GStreamerQuick-1.0	Qt5GStreamerQuick-1.0
Qt5GStreamerUi-1.0	Qt5GStreamerUi-1.0
Qt5GStreamerUtils-1.0	Qt5GStreamerUtils-1.0
Qt5Location	Qt5Location
Qt5Positioning	Qt5Positioning
Qt5Multimedia	Qt5Multimedia
Qt5MultimediaQuick_p	Qt5MultimediaQuick_p
Qt5MultimediaWidgets	Qt5MultimediaWidgets
Qt5Declarative	Qt5Declarative
Qt5Script	Qt5Script
Qt5ScriptTools	Qt5ScriptTools
Qt5Sensors	Qt5Sensors
Qt5SerialPort	Qt5SerialPort
Qt5TextToSpeech	Qt5TextToSpeech
Qt5Svg	Qt5Svg
Qt5CLucene	Qt5CLucene
Qt5Designer	Qt5Designer
Qt5DesignerComponents	Qt5DesignerComponents
Qt5Help	Qt5Help
Qt5Compositor	Qt5Compositor
Qt5WaylandClient	Qt5WaylandClient
Qt5WebChannel	Qt5WebChannel
Qt5WebKit	Qt5WebKit
Qt5WebKitWidgets	Qt5WebKitWidgets
Qt5WebSockets	Qt5WebSockets
Qt5X11Extras	Qt5X11Extras
Qt5XmlPatterns	Qt5XmlPatterns

LAPACK lapack
LibSSH libssh
LibXml2	xml2
Phonon phonon
Volk	volk
