package RPM::Source::Dependency::Analyzer::Collector::Plugin::Perl;

use strict;
use warnings;
use Pod::Strip;
use RPM::Source::Dependency::Analyzer::PerlUtils qw(_perl_pkg2dep _perl_req2dep);
use RPM::Source::Dependency::Analyzer::Collector::Parser;

our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser/;
our $verbose=0;

my %KNOWN_DBD_DRIVERS=map {$_=>1} qw/
DBD::Adabas
DBD::ADO
DBD::Amazon
DBD::AnyData
DBD::ASAny
DBD::Cego
DBD::Chart
DBD::CSV
DBD::DashProfiler
DBD::DB2
DBD::DBM
DBD::DBMaker
DBD::drizzle
DBD::DtfSQLmac
DBD::Empress
DBD::EmpressNet
DBD::ExampleP
DBD::Excel
DBD::File
DBD::Firebird
DBD::Fulcrum
DBD::Gofer
DBD::Google
DBD::Illustra
DBD::Informix
DBD::Informix4
DBD::Ingres
DBD::IngresII
DBD::InterBase
DBD::iPod
DBD::JDBC
DBD::LDAP
DBD::Log
DBD::MaxDB
DBD::Mimer
DBD::Mock
DBD::monetdb
DBD::mSQL
DBD::Multi
DBD::Multiplex
DBD::mysql
DBD::mysqlPP
DBD::NET
DBD::NullP
DBD::ODBC
DBD::Oracle
DBD::Ovrimos
DBD::PassThrough
DBD::Pg
DBD::PgLite
DBD::PgPP
DBD::PgSPI
DBD::Plibdata
DBD::pNET
DBD::PO
DBD::PrimeBase
DBD::Proxy
DBD::QBase
DBD::RAM
DBD::RDB
DBD::RDFStore
DBD::Redbase
DBD::Safe
DBD::Salesforce
DBD::SearchServer
DBD::Simulated
DBD::Solid
DBD::Sponge
DBD::Sprite
DBD::SQLAnywhere
DBD::Sqlflex
DBD::SqlflexTest
DBD::SQLite
DBD::SQLite2
DBD::Sybase
DBD::Sys
DBD::Template
DBD::TemplateSS
DBD::Teradata
DBD::TimesTen
DBD::TreeData
DBD::TSM
DBD::Unify
DBD::Wire10
DBD::WMI
DBD::WTSprite
DBD::XBase
DBD::Yaswi
/;

my %raw_perl_ignore=map {$_=>1} qw/
5
5.005
v5
bytes
constant
feature
inc
integer
lib
locale
strict
utf8
vars
version
warnings
Carp
Cwd
Data::Dumper
File::Basename
File::CheckTree
File::Compare
File::Copy
File::Find
File::Glob
File::Path
File::Spec
File::Temp
Getopt::Long
Getopt::Std
POSIX
Tie::Array
Tie::Handle
Tie::Hash
Tie::Memoize
Tie::Scalar
/;

sub parse_file {
    my ($self,$file)=@_;
    my $storage=$self->{-storage};

    # generated files (\.PL) and files that are 'use'd. but don't have its own package
    if ($file=~m!lib/(.*\.pm)(?:\.PL)?$!) {
	$storage->add_perl_ignore(&_perl_req2dep($1));
    }

    my $pod_stripper=Pod::Strip->new;              # create parser
    my $podless;                        # set output string
    $pod_stripper->output_string(\$podless);       # see Pod::Simple

    open my $fh, "<", $file or Carp::confess($!);
    $pod_stripper->parse_file($fh);
    close $fh;

    my $base_multiline;
    my $line_count=0;
    local $_;
    while (length($podless)) {
	if ($podless=~s/^([^\n]*)\n//) {
	    $_=$1;
	} else {
	    $_=$podless;
	    $podless='';
	}
	unless ($line_count) {
	    ##!/usr/bin/perl -MDevel::Cover
	    if (s,^#!/usr/bin/perl\s*,,) {
		my @mopts=grep{/^-M/} split(/\s+/,$_);
		map {s,-M,,} @mopts;
		map {$storage->add_perl_req( &_perl_pkg2dep($_) )} @mopts;
	    }
	}
	$line_count++;
	last if /^\s*__END__\s*;?\s*$/;

	if (defined $base_multiline) {
	    if (/^([^;]*);/) {
		$base_multiline.=' '.$1;
		&__perl_process_use_base($base_multiline,$storage);
		$base_multiline=undef;
	    } else {
		$base_multiline.=' '.$_;
	    }
	    next;
	}

	# experimental.
	if (/[\W]dbi:([a-zA-Z]\w*):/ and $KNOWN_DBD_DRIVERS{'DBD::'.$1}) {
	    $storage->add_perl_req( &_perl_pkg2dep('DBD::'.$1) );
	}

	if (/^\s*use\s+([\w\d_:]+)/ and not $raw_perl_ignore{$1}) {
	    my $module=$1;
	    if ($module eq 'base' or $module eq 'parent') {
		$storage->add_perl_req( &_perl_pkg2dep($module) );
		my $base=$_;
		$base=~s/^\s*use\s+(?:base|parent)\s+//;
		if ($base=~s/\s*;\s*$//) {
		    &__perl_process_use_base($base,$storage);
		} else { 
		    $base_multiline=$_;
		    next;
		}
	    } else {
		# something like pod - still slips in :(
		# use a C<tie>-based interface to use the standard Perl I/O operators.
		$storage->add_perl_req( &_perl_pkg2dep($module) ) if not /^\s*use\s+[\w\d_:]+\s+\w+\s+\w+\s/;
	    }

	# with, extends (Moo/Mouse/Moose OOP)
	# with qw(Data::Stream::Bulk); see Data-Stream-Bulk-AnyEvent-v0.0.2
	} elsif (m/^\s*(?:with|extends)\s*qw['\[\]!(){}|\/]([\w\d_:\s]+)['\[\]!(){}|\/]\s*;\s*$/) {
	    $storage->add_perl_req( map{&_perl_pkg2dep($_)} split (/\s+/,$1) );
	} elsif (m/^\s*(?:with|extends)\s*(?:\(\s*)?['"]([^\)]*)['"](?:\)\s*)?\s*;\s*$/) {
	    $storage->add_perl_req( map{&_perl_pkg2dep($_)} split (/['"]\s*,\s*['"]/,$1) );

	} elsif (m!^\s*use_ok\s*\(\s*['"]([\w_:]+)['"]\s*\)\s*;!) {
	    my $module=$1;
	    $storage->add_perl_req( &_perl_pkg2dep($module) );
	} elsif (m!(?:^|\s|\{|;)require\s+['"]([\w\d_/\.]+)['"]\s*;!) {
	    # TODO: CHECK If softreq (prev.line is eval {)
	    $storage->add_perl_req( &_perl_req2dep($1) );
	} elsif (m!(?:^|\s|\{|;)require\s+([\w:]+)\s*;!) {
	    # TODO: CHECK If softreq (prev.line is eval {)
	    $storage->add_perl_req( &_perl_pkg2dep($1) );
	} elsif (/^\s*package\s+([\w\d_:]+)/) {
	    my $module=$1;
	    $storage->add_perl_ignore( &_perl_pkg2dep($module) ) if not /^\s*package\s+[\w\d_:]+\s+\w+\s+\w+\s/;
	} elsif (/^__(?:END|DATA)__/) {
	    last;
	}
    }
}

sub __perl_process_use_base {
    my ($base,$storage)=@_;
    # TODO: should not go there ...
    $base=~s/^\s*//;
    #use base 'Exporter'; our @EXPORT = qw(uninstall);
    $base=~s/;\s*our\s.*$//;
    # clean up comments
    $base=~s/;\s*#.*$//;

        #use parent -norequire, qw(perl5i::0::DateTime::y2038);
    if ($base=~m/^-norequire/) {
	# norequire: do nothing, internal dep
    } elsif ($base=~m/^qw['\[\]!(){}|\/]([\w\d_:\s]+)['\[\]!(){}|\/]/) {
	$storage->add_perl_req( map{&_perl_pkg2dep($_)} split (/\s+/,$1) );
	# use base 'Class::Accessor';
    } elsif ($base=~m/^(?:\(\s*)?['"]([^\)]*)['"]\s*(?:\)\s*)?$/) {
	$storage->add_perl_req( map{&_perl_pkg2dep($_)} split (/['"]\s*,\s*['"]/,$1) );
	# use base Class::Accessor;
    } elsif ($base=~m/^([\w\d_:]+)/) {
	$storage->add_perl_req( map{&_perl_pkg2dep($_)} $1 );
    } else {
	print STDERR "TODO:SourceAnalyzer:use_base: can't parse [$base]:$_\n";
    }
}


1;
