package RPM::Source::Dependency::Analyzer::Resolver;

use Carp;
use strict;
use warnings;
use DistroMap 0.39;
# 10 = python
$DistroMap::Repository::opt_distrodb_version=11;

use RPM::Source::Tools::BuildReqStorage 0.101;
use RPM::Source::Dependency::Analyzer::PerlUtils qw(_perl_pkg2dep _perl_req2dep);
use RPM::Source::Dependency::Analyzer::Resolver::SingleTypeFactory;
use Source::Shared::CLI;
our @ISA=qw/Source::Shared::CLI/;

our $VERSION=0.016002;

### TODO:
# push mapped name [packages] to special collector and check them against buildreq again 
# to prevent duplicates like
#BuildRequires: libSDL-devel /usr/bin/sdl-config

#### TODO:
#### drop it and replace with resolver option(s) about multiple providers: keep, ask, guess?
use RPM::Source::Dependency::HackOverSourceAnalyzer;

our ($opt_lookup_rebased_headers);
our ($opt_disable_python,$opt_resolve_python,$opt_allow_unknown_python,$opt_keep_explicit_python);
our ($opt_disable_pkgconfig,$opt_resolve_pkgconfig,$opt_allow_unknown_pkgconfig,$opt_keep_explicit_pkgconfig);
our ($opt_disable_perl,$opt_resolve_perl,$opt_allow_unknown_perl,$opt_keep_explicit_perl);
our ($opt_disable_path,$opt_resolve_path,$opt_allow_unknown_path,$opt_keep_explicit_path);
our %opt_select_multiple_dependency;
my (@opt_resolve_context_raw,$opt_resolve_context);

# experimental, not API; use
# --SET RPM::Source::Dependency::Analyzer::Resolver::opt_qt_version=5
# TODO: add support for multiple QT's, move to context
our $opt_qt_version;

our $verbose=1;

our @LONGOPT=(
    'sourcedep-lookup-rebased-headers'   => \$opt_lookup_rebased_headers,
    'sourcedep-pkg-resolve=s' => sub {&__setopt('resolve',$_[1])},
    'sourcedep-allow-unknown=s' => sub {&__setopt('allow_unknown',$_[1])},
    'sourcedep-keep-explicit=s' => sub {&__setopt('keep_explicit',$_[1])},
    'sourcedep-resolver-disable=s'  => sub {&__setopt('disable',$_[1])},
    'sourcedep-context=s' => \@opt_resolve_context_raw,
    'sourcedep-select-dependency=s%' => \%opt_select_multiple_dependency,
    );

sub print_longopt {
    print "SourceAnalyzer::DependencyResolver Options:
    Options for special dependencies management.
      Special dependencies are dependencies like type(name).
      They are often stricter than dependencies on package names.
      Generation of such dependencies is ruled by options --sourcedep-*.
	--sourcedep-pkg-resolve=<types>		replace all special dependencies with package names.
	--sourcedep-keep-explicit=<types>	keep special dependencies even if their packages are mentioned in the spec.
	--sourcedep-resolver-disable=<types>	ignore specified dependencies.
	--sourcedep-allow-unknown=<types>	keep all special dependencies, even not found in the repository.
	<types> is a comma separated list of possible types and metacommands:
		 all,none,path,perl,pkgconfig,nopath,noperl,nopkgconfig.
      Examples:
	--sourcedep-pkg-resolve=pkgconfig,perl	use package names instead of pkgconfig(name) or perl(Name.pm)
	--sourcedep-pkg-resolve=all		always use package names
	--sourcedep-pkg-resolve=nopython	use pythonX.Y(name) instead of package names
	--sourcedep-keep-explicit=pkgconfig	skip pkgconfig(name) dependency if its package is present (default).
	--sourcedep-allow-unknown=pkgconfig	add pkgconfig(name) dependency even if its package is not found.
	--sourcedep-resolver-disable=perl	do not generate perl dependencies.
      Context for ambiguous dependency resolution
	--sourcedep-context=<list>		resolve dependendencies for given context
      Examples:
	--sourcedep-context=python3	resolve python dependendencies in python3
	--sourcedep-context=python2,python3	resolve python dependendencies in both python2 and python3
    Options for selecting a package among multiple candidates.
	--sourcedep-select-dependency name=package	Select explicit package if the dependency <name> is provided by several packages.
";
}

my %KNOWN_MAPPABLE=map{$_=>1} qw/path perl pkgconfig python/;
my %KNOWN_CONTEXT=map{$_=>1} qw/python2 python3/;

sub check_context {
    my $in_context=shift;
    return unless $in_context;
    confess "hash expected" unless ref($in_context);
    foreach my $key (keys(%$in_context)) {
	die "invalid --sourcedep-context: $key (expected: ",join('|',keys(%KNOWN_CONTEXT)),")\n" unless $KNOWN_CONTEXT{$key};
    }
}

sub process_options {
    $opt_resolve_context={ map{$_=>1} map {split(',',$_)} @opt_resolve_context_raw };
    &check_context($opt_resolve_context);
}

sub __setopt {
    my ($opt_body,$args)=@_;
    my %val;
    foreach my $arg (split(',',$args)) {
	if ($arg eq 'all') {
	    foreach my $key (keys (%KNOWN_MAPPABLE)) {
		$val{$key}=1;
	    }
	} elsif ($arg eq 'none') {
	    foreach my $key (keys (%KNOWN_MAPPABLE)) {
		$val{$key}=0;
	    }
	} elsif ($KNOWN_MAPPABLE{$arg}) {
	    $val{$arg}=1;
	} elsif ($KNOWN_MAPPABLE{'no'.$arg}) {
	    $val{$arg}=0;
	} else {
	    die "failed to process ",&__optbody2opt($opt_body),": unknown arg $arg";
	}
    }
    foreach my $key (keys (%val)) {
	my $val=$val{$key};
	my $evalstring='$opt_'.$opt_body.'_'.$key."='$val'";
	eval $evalstring;
	die 'Oops! internal error. failed to assign ',$evalstring,' for ',&__optbody2opt($opt_body),":$@\nPlease, report." if $@;
    }
}

sub __optbody2opt {
    my $optstring='--sourcedep-'.$_[0];
    $optstring=~s/_/-/g;
    return $optstring;
}

sub new {
    my $class = shift;
    my $self = {
	-verbose => $verbose,

	-disable_python => $opt_disable_python,
	-resolve_python => $opt_resolve_python,
	-allow_unknown_python => $opt_allow_unknown_python,
	-keep_explicit_python => $opt_keep_explicit_python,

	-disable_pkgconfig => $opt_disable_pkgconfig,
	-resolve_pkgconfig => $opt_resolve_pkgconfig,
	-allow_unknown_pkgconfig => $opt_allow_unknown_pkgconfig,
	-keep_explicit_pkgconfig => $opt_keep_explicit_pkgconfig,

	-disable_perl => $opt_disable_perl,
	-resolve_perl => $opt_resolve_perl,
	-allow_unknown_perl => $opt_allow_unknown_perl,
	-keep_explicit_perl => $opt_keep_explicit_perl,

	-disable_path => $opt_disable_path,
	-resolve_path => $opt_resolve_path,
	-allow_unknown_path => $opt_allow_unknown_path,
	-keep_explicit_path => $opt_keep_explicit_path,

	# TODO: something like (rebased filelist) '^examples/'
	#-skip_analyze_by_pattern => $opt_skip_analyze_by_pattern,

	-lookup_rebased_headers => $opt_lookup_rebased_headers,

	-resolve_context => $opt_resolve_context,
	-disable_postresolve => 0,

	-select_multiple_dependency => {},
	&RPM::Source::Dependency::HackOverSourceAnalyzer::get_source_analyzer_hack_opts(),

	@_,
    };
    # todo: check context again? bad for BundleImport, I guess.
    #&check_context($self->{-resolve_context});
    bless $self, $class;
    return $self;
}

my %BIN_IGNORE=map {$_=>1} qw!!;
my %BIN_DROP=map {$_=>1} 
# devel utils found in rpm-build
qw!
]
make
gmake
pkg-config
ar
cc
cl
gcc
gtar
ld
objcopy
ranlib
strip
lib.exe
csc.exe!,
#AC_CHECK_PROGS(xterm, Eterm gnome-terminal aterm rxvt xterm)
qw!Eterm!,
# can be mapped on /usr/bin/run in python3-module-runfile
qw!run!;


my %PKGCNF_DROP=map {$_=>1} 
# deprecated
qw!
hal
!;

my %LIB_DROP=map {$_=>1} # glibc stubs
qw!socket bsd nsl inet intl!;

my %LIB_IGNORE=map {$_=>1} 
# deprecated
qw!
sasl
!, # in std lang environment
qw!
gcc
pthread
stdc++
!, #other_unix
'gen', # libgen -_string_pattern-matching_library_Solaris
qw!
getopt
getopt_long
gnugetopt
socket
sunmath
mlib
!, # not built for Sisyphus
qw!
xcb-renderutil
xcb-xprint
!, # Must die
qw!
Advapi32
advapi32
bps
clipboard
clntsh
comdlg32
coredll
crypt32
d3d9
d3dcompiler
db2
db2cli
dnsapi
dxguid
gdi32
gds
gds32_ms
iconv
icuin
inet
intl
iphlpapi
kernel32
msimg32
netapi32
odbc32
ole32
oleacc
oleaut32
OpenAL32
openal32
opengl32
posix4
pps
qui
qtest
resmgr
rpcrt4
screen
sendfile
setupapi
slog2
sqldb
user32
WFD
winmm
winspool
wldap32
Ws2_32
ws2_32
ws2
ws32
wsock32
!;
my %HEADER_IGNORE=map {$_=>1} (
'Carbon/Carbon.h',	# MAC
'CoreServices/CoreServices.h', # MAC
'QuickTime/QuickTime.h', # MAC
'fp_class.h',		# *
'ieeefp.h',		# *
'machine/endian.h',	# BSD
'machine/types.h',	# BSD
'nan.h',		# BSD
'priv.h',		# Solaris
'sndio.h',		# OpenBSD,...
'sunmath.h',		# Solaris
'sys/byteswap.h',	# SPARC?
'sys/dg_sys_info.h',	# AIX, HP-UX
'sys/audioio.h',	# BSD
'sys/cdio.h',		# BSD
'sys/fbio.h',		# BSD
'sys/disk.h',		# BSD
'sys/endian.h',		# BSD
'sys/extaddr.h',	# BSD
'sys/if.h',		# BSD
'sys/int_types.h',	# BSD
'sys/mode.h',		# AIX
'sys/sockio.h',		# BSD
'sys/scsiio.h',		# BSD
'sys/videoio.h',	# BSD
'sys/systeminfo.h',	# Solaris
'sys/processor.h',	# Solaris
'sys/loadavg.h',	# Solaris
'sys/fpu.h',		# *
'windows.h',	# Must die
'winsock.h',	# Must die
'winsock2.h',	# Must die
'ws2tcpip.h',	# Must die
'basetsd.h',	# Must die
'hash_map',	# Must die
'hash_set',	# Must die
'iphlpapi.h',	# Must die
'tchar.h',	# Must die
'mmsystem.h',	# Must die
'DShow.h',	# DirectShow
'strstream',
),
# extra sys headers
qw!
android/native_window.h
cygwin/fs.h
machine/soundcard.h
socket.h
sys/acpa.h
sys/apparmor.h
sys/audio.h
sys/bufmod.h
sys/devpoll.h
sys/disklabel.h
sys/dl.h
sys/dlpi_ext.h
sys/dlpi.h
sys/dlpihdr.h
sys/dvdio.h
sys/event.h
sys/extattr.h
sys/fpu.h
sys/ioccom.h
sys/mib.h
sys/mixer.h
sys/mkdev.h
sys/mntent.h
sys/ndd_var.h
sys/net/nit.h
sys/proc.h
sys/ptyvar.h
sys/sam9407.h
sys/sched.h
sys/scsi.h
sys/scsi/scsi_types.h
sys/sdt.h
sys/stdtypes.h
sys/stropts.h
sys/telephony.h
sys/ttycom.h
sys/uc_access.h
sys/ucred.h
sys/utime.h
sys/videodev2.h
!;
my %HEADER_DROP=map {$_=>1} qw!
config.h
!;

my %GIR_IGNORE=map {$_=>1} qw//;
my %PERL_IGNORE=map {$_=>1} qw!
______misc_____________________
perl(Win32/API.pm)
perl(v018.nofile2.pl)
!;
my %PERL_DROP=map {$_=>1} qw!
perl(File/Spec/Win32.pm)
perl(ExtUtils/MM_AIX.pm)
perl(ExtUtils/MM_BeOS.pm)
perl(ExtUtils/MM_Cygwin.pm)
perl(ExtUtils/MM_Darwin.pm)
perl(ExtUtils/MM_DOS.pm)
perl(ExtUtils/MM_MacOS.pm)
perl(ExtUtils/MM_NW5.pm)
perl(ExtUtils/MM_OS2.pm)
perl(ExtUtils/MM_QNX.pm)
perl(ExtUtils/MM_UWIN.pm)
perl(ExtUtils/MM_VMS.pm)
perl(ExtUtils/MM_VOS.pm)
perl(ExtUtils/MM_Win32.pm)
perl(ExtUtils/MM_Win95.pm)
______generated_modules________
perl(Marpa/XS/Version.pm)
perl(XML/SAX/ExpatXS/Preload.pm)
!;

my %CMAKE_IGNORE=map {$_=>1} (
'IOKit', # MAC
'Qt5PlatformSupport', # static lib
'Qt5WinExtras', # WIN
);

my %QMAKE_IGNORE=map {$_=>1} (
'platformsupport', # static lib
'winextras', # WIN
);


# TODO
#my %IGNORE=(
#'gir' => \%GIR_IGNORE,
#'devel-libs' => \%LIB_IGNORE,
#'perl' => \%PERL_IGNORE,
#'headers' => \%HEADER_IGNORE,
#'pkg-config' => {},
#    );

##### TODO!!!! should be in distrodb !!!! ####
&__add_file_to_flag('/usr/share/RPM-Source-Dependency-Analyzer/headers-ignore/stdheaders.txt',\%HEADER_IGNORE);
##############################################

sub __add_file_to_flag {
    my ($file,$hash)=@_;
    if (! -e $file && $ENV{'RPM_BUILD_ROOT'}) {
	# for tests: load from .
	$file=~s!^.*/!!;
	($file) = glob ($ENV{'RPM_BUILD_DIR'}."/*/$file");
    }
    die "Oops! can't load $file" unless -e $file;
    open my $fh, "<", $file or die $!;
    local $_;
    while (<$fh>) {
	chomp;
	s/^\s*//;
	s/\s*$//;
	$hash->{$_}=1;
    }
    close $fh;
}

# TODO:
my %HEADER2PKG=qw/
jni.h	java-devel-default
/;

### static method
sub is_arch_dependent {
    my (undef,$storage)=@_;
    # lang_analyze_storage
    foreach my $bin_lang (qw/c cxx fortran swig perl_xs/) {
	if ($storage->get_lang_flag($bin_lang)) {
	    return 1;
	    last;
	}
    }
    return 0;
}

sub resolve_storage {
    my ($self,$storage,$buildreq)=@_;
    $self->resolve(
	-storage => $storage,
	-buildreq => $buildreq,
	);
}

sub resolve {
    my $self=shift;
    my %args=(
	# -buildreq_oldauto => undef,
	-distroinfo => $self->{-distroinfo},
	-verbose => $self->{-verbose},
	-storage => $self->{-storage},
	-buildreq => $self->{-buildreq},
	-spec => $self->{-spec},
#	- => $self->{-},
	@_,
	);
    my $verbose=&__max($self->{verbose},$args{-verbose});
    my $inputname=$args{-inputname};
    my $spec=$args{-spec};

    my $distroinfo=$args{-distroinfo};
    my $storage=$args{-storage};
    my $buildreq=$args{-buildreq};

    Carp::croak "-storage is required" unless $storage;
    Carp::croak "-buildreq is required" unless $buildreq;
    Carp::croak "-distroinfo is required" unless $distroinfo;

    $storage->_uniquify_storage();
    $storage->_debug_storage('resolve_storage (uniquified)') if $verbose>1;

    my $buildreq_existing=$buildreq;
    # support for parent in Convert
    my $buildreq_oldauto=$self->{-buildreq_oldauto};
    if ($spec) {
	my $spec_current_state=$buildreq->collect_spec_current_deps_and_sourcedep_entry($spec);
	my $existing=$spec_current_state->{-manual_buildreq};
	my $existing_pre=$spec_current_state->{-manual_buildreq_pre};
	my $oldauto=$spec_current_state->{-sourcedep_entry_buildreq};
	my $oldauto_pre=$spec_current_state->{-sourcedep_entry_buildreq_pre};
	$buildreq_existing=$buildreq->clone();
	$buildreq_existing->add_buildreq(keys %$existing);
	$buildreq_existing->add_pre_buildreq(keys %$existing_pre);

	$buildreq_oldauto=RPM::Source::Tools::BuildReqStorage->new_from_br_brpre($oldauto,$oldauto_pre)
	    if not $buildreq_oldauto and (scalar %$oldauto or scalar %$oldauto_pre);
    }

    my @IGNORE=$storage->get_pkgs_ignore;
    my @BR=$storage->get_pkgs_req;
    my @MAPPED_BR;
    my $flag_not_found;
    my $context=$self->{-resolve_context};

    my $factory=RPM::Source::Dependency::Analyzer::Resolver::SingleTypeFactory->new(
        -verbose => $verbose,
        -resolverobj => $self,
        -buildreqobj => $buildreq_existing,
        -buildreq_oldauto_obj => $buildreq_oldauto,
        -distroinfoobj => $distroinfo,
        -storageobj => $storage,
        -flag_not_found_ref => \$flag_not_found,
        -resolved_br_ref => \@BR,
        -mapped_br_ref => \@MAPPED_BR,
        -resolved_ignore_ref => \@IGNORE,
	);

    my @resolvers=(
	$factory->create(
	    -distrodb_name => 'devel-libs',
	    -IGNORE_NOT_FOUND_hash => \%LIB_IGNORE,
	    -DROP_hash => \%LIB_DROP,
	),
	$factory->create(
	    -distrodb_name => 'headers',
	    -IGNORE_NOT_FOUND_hash => \%HEADER_IGNORE,
	    -DROP_hash => \%HEADER_DROP,
	),
	$factory->create(
	    -distrodb_name => 'gir',
	    -IGNORE_NOT_FOUND_hash => \%GIR_IGNORE,
	),
	);
    push @resolvers,
	$factory->create(
	    -class => 'RPM::Source::Dependency::Analyzer::Resolver::CMakeResolver',
	    -distrodb_name => 'cmake',
	    -IGNORE_NOT_FOUND_hash => \%CMAKE_IGNORE,
	) if $storage->has_cmake_req;
    push @resolvers,
	$factory->create(
	    -distrodb_name => 'pkg-config',
	    -DROP_hash => \%PKGCNF_DROP,
	    -is_mappable => 1,
	    -mappable_optsuffix => 'pkgconfig',
	    -mapper => sub { return "pkgconfig($_[0])"},
	) if !$self->{-disable_pkgconfig} and $storage->has_pkgconf_req;
    push @resolvers,
	$factory->create(
	    -distrodb_name => 'perl',
	    -IGNORE_NOT_FOUND_hash => \%PERL_IGNORE,
	    -DROP_hash => \%PERL_DROP,
	    -is_mappable => 1,
	    -mappable_optsuffix => 'perl',
	) if !$self->{-disable_perl} and $storage->has_perl_req;
    push @resolvers,
	$factory->create(
	    -class => 'RPM::Source::Dependency::Analyzer::Resolver::PathResolver',
	    -distrodb_name => 'path',
	    -IGNORE_NOT_FOUND_hash => \%BIN_IGNORE,
	    -DROP_hash => \%BIN_DROP,
	    -is_mappable => 1,
	    -mappable_optsuffix => 'path',
	) if !$self->{-disable_path} and $storage->has_path_req;

    if ($self->{-lookup_rebased_headers} and $storage->has_hdrs_req) {
	# TODO: experimental, off by default
	push @resolvers,
	$factory->create(
	    -distrodb_name => 'headers-rebased',
	    -IGNORE_NOT_FOUND_hash => \%HEADER_IGNORE,
	    -DROP_hash => \%HEADER_DROP,
	    );
    }
    if ($storage->has_qt_component_req) {
	my $qt_version=$opt_qt_version;
	#warn "has qt_component_req:", join(',',$storage->get_qt_component_req);
	# TODO: determine context by existing br and by matching qt_component to qt4 and 5
	if ($qt_version) {
	    push @resolvers,
		$factory->create(
		    -storage_name => 'qt_component',
		    -IGNORE_NOT_FOUND_hash => \%QMAKE_IGNORE,
		    -class => 'RPM::Source::Dependency::Analyzer::Resolver::QMakeResolver',
		    -target => $qt_version,
		);
	}
    }
    if (!$self->{-disable_python} and $storage->has_python_req) {
	push @resolvers, $factory->create(
	    -class => 'RPM::Source::Dependency::Analyzer::Resolver::PythonResolver',
	    -distrodb_name => 'python2',
	    ) if $context->{python2};
	push @resolvers, $factory->create(
	    -class => 'RPM::Source::Dependency::Analyzer::Resolver::PythonResolver',
	    -distrodb_name => 'python3',
	    ) if $context->{python3};
    }
    map {$_->resolve()} @resolvers;

    my (@SUBST_BR,@SUBST_IGNORE);
    foreach my $br (@BR) {
	my @data=$distroinfo->translate('buildreqs-subst',$br);
	if (@data) {
	    push @SUBST_BR, @data;
	} else {
	    push @SUBST_BR, $br;
	}
    }
    foreach my $ign (@IGNORE) {
	push @SUBST_IGNORE, $ign, $distroinfo->translate('buildreqs-subst',$ign);
    }

    @BR = sort &__uniquify_list(@SUBST_BR);
    print STDERR "resolve_storage: BR(detected):", join(' ',@BR),"\n" if $verbose >1;
    if ($inputname) {
	$buildreq->add_buildreq_for($inputname,@BR);
    } else {
	$buildreq->add_buildreq(@BR);
    }
    @IGNORE = sort &__uniquify_list(@SUBST_IGNORE);
    print STDERR "resolve_storage: Ignore BR:", join(' ',@IGNORE),"\n" if $verbose >1;
    $buildreq->ignore_buildreq(@IGNORE);

    foreach my $mapped_br (@MAPPED_BR) {
	my ($mapped_name, $dataref) =@$mapped_br;
	$buildreq->add_buildreq($mapped_name) if ! &__rr_buildreq_have_any($buildreq,$dataref);
    }

    warn "SourceAnalyzer: some deps weren't found. Is DistroMap database outdated?\n" if $flag_not_found and $verbose;

    &postresolve_call($storage,$buildreq,$context) if ! $self->{-disable_postresolve};
}

### TODO: this hack resolving is really ALT-specific --- :(
### we need a way to write it in distro-independent way
sub postresolve_call {
    my ($storage,$buildreq,$context)=@_;
    $buildreq->add_buildreq('swig') if $storage->get_lang_flag('swig');
    $buildreq->add_buildreq('gcc-c++') if $storage->get_lang_flag('cxx');
    $buildreq->add_buildreq('gcc-fortran') if $storage->get_lang_flag('fortran');
    $buildreq->add_buildreq('perl-devel') if $storage->get_lang_flag('perl_buildscripts');
    # not a usr/bin :(
    $buildreq->add_buildreq('texinfo') if $storage->get_lang_flag('texinfo');
    ### TODO convert to (post) storage hacks
    $buildreq->add_buildreq('/usr/bin/pod2man') if $storage->get_lang_flag('prog_pod2man');
    $buildreq->add_buildreq(&_perl_pkg2dep('CPAN')) if $storage->get_lang_flag('perl_inc_Module_Install');
    $buildreq->del_buildreq(&_perl_pkg2dep('CPANPLUS::Backend')) if $storage->get_lang_flag('perl_inc_Module_AutoInstall');
    # python
    if ($storage->get_lang_flag('python')) {
	if ($context->{python2}) {
	    $buildreq->add_buildreq('python-devel');
	}
        #if ($context->{python3}) {
	$buildreq->add_buildreq('python3-devel');
    }
    # end lang

    ## known ignore hacks: TODO distrodb-ize
    $buildreq->ignore_buildreq(qw/python-modules-encodings python-modules-curses python-modules-xml python-modules-compiler python-modules-hotshot python-modules-bsddb python-modules-logging/) if $buildreq->has_buildreq('python');
    $buildreq->ignore_buildreq('python-base') if $buildreq->has_buildreq('python-devel');
    $buildreq->ignore_buildreq('python-dev') if $buildreq->has_buildreq('python-devel');
    $buildreq->ignore_buildreq('python') if $buildreq->has_buildreq('python-devel');
    $buildreq->ignore_buildreq('python') if $buildreq->has_buildreq('python-dev');
    $buildreq->ignore_buildreq('python3-base') if $buildreq->has_buildreq('python3-devel');
    $buildreq->ignore_buildreq('python3-dev') if $buildreq->has_buildreq('python3-devel');
    $buildreq->ignore_buildreq('python3') if $buildreq->has_buildreq('python3-devel');
    $buildreq->ignore_buildreq('python3') if $buildreq->has_buildreq('python3-dev');
    if ($buildreq->has_buildreq('/usr/bin/python')) {
      LOOP_PYBR:
	foreach my $pydepbr (qw!pkgconfig(python3) pkgconfig(python2) python-devel python3-devel /usr/bin/python2 /usr/bin/python3!) {
	    if ($buildreq->has_buildreq($pydepbr)) {
		$buildreq->ignore_buildreq('/usr/bin/python');
		last LOOP_PYBR;
	    }
        }
    }
    $buildreq->ignore_buildreq('perl') if $buildreq->has_buildreq('perl-devel');
    $buildreq->ignore_buildreq('perl-base') if $buildreq->has_buildreq('perl-devel');
    $buildreq->ignore_buildreq('perl','perl-base','perl-devel') if $buildreq->has_buildreq('rpm-build-perl');
#cfitsio: # BEGIN SourceDeps(oneline): BuildRequires: /usr/bin/gfortran
#/usr/bin/gfortran по сути избыточный - в спеке уже есть gcc-fortran,
#но gcc-fortran (предоставляется не одним пакетом, а многими, это библиотеку сбило с толку)
    $buildreq->ignore_buildreq('/usr/bin/fortran') if $buildreq->has_buildreq('gcc-fortran');
    ## 

    # TODO: interdeps table
    # TODO: can also be in DependencyCollector. move?
    $buildreq->add_buildreq(&_perl_pkg2dep('Moose')) if $buildreq->has_buildreq(&_perl_pkg2dep('Any::Moose'));
    $buildreq->add_buildreq(&_perl_pkg2dep('Test::Most')) if $buildreq->has_buildreq(&_perl_pkg2dep('Test::Class::Most'));
    # fixed in package
    #$buildreq->add_buildreq(&_perl_pkg2dep('JSON::XS')) if $buildreq->has_buildreq(&_perl_pkg2dep('JSON::MaybeXS'));

    # TODO: add explicit cleanup interface
    #foreach my $dep ($buildreq->get_list_buildreq()) {
    #	#perl-Module-Install-1.06-alt1.noarch provides perl(inc/Module/Install.pm)
    #   $buildreq->del_buildreq($dep) if $dep=~m!^perl(inc/!;
    #}
}


sub __rr_buildreq_have_any {
    my ($buildreq,$list)=@_;
    foreach my $key (@$list) {
	Carp::cluck "__rr_buildreq_have_any: got undef" unless defined $key;
	return 1 if $buildreq->has_buildreq($key);
	return 1 if $buildreq->has_pre_buildreq($key);
	return 1 if $buildreq->has_ignore_buildreq($key);
	#return 1 if $self->is_already_present_pkg($key);
    }
    return 0;
}

sub __uniquify_list {
    my %req;
    foreach my $a (@_) {
	if (not defined $a) {
	    Carp::cluck "undef in __uniquify_list";
	} elsif (ref($a)) {
	    Carp::cluck "bad ref in __uniquify_list: $a";
	} else {
	    $req{$a}=1;
	}
    }
    return keys %req;
}

sub __max {
    my $max=shift;
    $max//=0;
    foreach my $arg (@_) {
	$max=$arg if defined($arg) && $arg>$max;
    }
    return $max;
}

1;
