package RPM::Source::Dependency::Analyzer::Storage;

use strict;
use warnings;
use Carp;

#    version  maj:min
our $VERSION=0.006000;

our $verbose=0;

my @_DEPS_KEYS=qw/
cmake girs hdrs libs lua path perl pkgconf pkgs python qt_component ruby vapi typelib
/;
my %SA_STORAGE_KEY=(
'gir','girs',
'vapi','vapi',
'typelib','typelib',
'devel-libs','libs',
'headers','hdrs',
'headers-rebased','hdrs',
'lua','lua',
'packages', 'pkgs',
'path','path',
'perl','perl',
'pkg-config','pkgconf',
'python2','python',
'python3','python',
'ruby','ruby',
'cmake','cmake',
);
my %_DEBUG_NAMES=qw/
libs devel-libs
hdrs headers
path executables
pkgs packages
girs gir-files
pkgconf pkgconfig
/;

my %known_lang=map{$_=>1} qw/
c
cxx
fortran
go
lua
perl_xs
perl_buildscripts
perl_inc_Module_AutoInstall
perl_inc_Module_Install
prog_pod2man
python
qmake
ruby
swig
texinfo
/;

sub new {
    my $class = shift;
    my @key_storages=map {
	$_.'_ignore' => [],
	$_.'_req' => []
    } @_DEPS_KEYS;

    my $self = {
	@key_storages,
	lang=>{},
	@_
    };
    bless $self, $class;
}

sub dump {
    my ($storage)=@_;
    my $sorted=$storage->__create_dump_arrayref();
    map {
    print $_->[0],"\t",$_->[1],"\t",$_->[2],"\n";
    } @$sorted;
}

sub dump_to_file {
    my ($storage,$filename)=@_;
    my $sorted=$storage->__create_dump_arrayref();
    open (my $fn, '>', $filename) || die "dump_to_file('$filename'): $!. Stopped";
    map {
    print $fn $_->[0],"\t",$_->[1],"\t",$_->[2],"\n";
    } @$sorted;
    close ($fn) || die "dump_to_file('$filename'): $!. Stopped";
}

sub __create_dump_arrayref {
    my ($storage)=@_;
    $storage->_uniquify_storage();
    my @text_to_sort;
    foreach my $lang ( keys(%{$storage->{lang}}) ) {
	my $val=$storage->{lang}->{$lang};
	push @text_to_sort, ['langflag',$lang, $val ? '1' : '0'];
    }
    foreach my $deproot (@_DEPS_KEYS) {
	foreach my $type ('ignore','req') {
	    my $key =$deproot.'_'.$type;
	    my $typestring=$type eq 'ignore'? $type: 'require';
	    foreach my $val (@{$storage->{$key}}) {
		if (ref $val) {
		    my $text='['.join(':',sort @$val).']';
		    push @text_to_sort, [$typestring,$deproot,$text];
		} else {
		    push @text_to_sort, [$typestring,$deproot,$val];
		}
	    }
	}
    }
    my @sorted=sort {
	my $cmp=$a->[0] cmp $b->[0];
	$cmp=$a->[1] cmp $b->[1] if $cmp==0;
	$cmp=$a->[2] cmp $b->[2] if $cmp==0;
	return $cmp;
    } @text_to_sort;
    return \@sorted;
}

sub _uniquify_storage {
    my ($storage)=@_;
    foreach (@_DEPS_KEYS) {
	foreach my $key ($_.'_ignore',$_.'_req') {
	    my @new_list=&__unique_list(@{$storage->{$key}});
	    $storage->{$key}=\@new_list;
	}
    }
}

sub __unique_list {
    # array for alternative choices
    my @out;
    my %req;
    foreach my $a (@_) {
	if (not defined $a) {
	    Carp::cluck "undef in __unique_list";
	} elsif (ref($a)) {
	    if (ref($a) eq 'ARRAY') {
		if (@$a == 0) {
		    Carp::carp "Oops! empty ref in __unique_list";
		} elsif (@$a == 1) {
		    # unarray it
		    $req{$a->[0]}=1;
		} else {
		    push @out, $a;
		}
	    } else {
		Carp::cluck "bad ref in __unique_list: $a";
	    }
	} else {
	    $req{$a}=1;
	}
    }
    return @out, keys %req;
}

sub distrodb_name2storage_name {
    return $SA_STORAGE_KEY{$_[1]};
}

sub __DEBUG_NAMES {
    my $debugname=$_DEBUG_NAMES{$_[0]};
    return $debugname ? $debugname : $_[0];
}

sub _debug_storage {
    my ($storage,$inputname)=@_;
    foreach my $dbkey (@_DEPS_KEYS) {
	&__debug_msg($inputname,'BR '.&__DEBUG_NAMES($dbkey),$storage->{$dbkey.'_req'});
    }
    foreach my $dbkey (@_DEPS_KEYS) {
	&__debug_msg($inputname,'ignore '.&__DEBUG_NAMES($dbkey),$storage->{$dbkey.'_ignore'});
    }
}

sub __debug_msg {
    my ($file,$br_name,$list)=@_;
    return unless @$list;
    print STDERR "$file: $br_name(detected):",
    join(' ',map {ref($_) eq 'ARRAY' ? '[ '.join(' ',@$_).' ]' : $_} @$list),"\n";
}

sub get_lang_flag {
    my ($self,$lang)=@_;
    Carp::confess "unsupported lang $_[1]" unless $known_lang{$_[1]};
    print STDERR "DEBUG:Collect: add_XXX_req: ",join(',',@_),"\n" if $verbose;
    return $self->{'lang'}->{$lang};
}
sub set_lang_flag {
    my ($self,$lang)=@_;
    Carp::confess "unsupported lang $_[1]" unless $known_lang{$_[1]};
    print STDERR "DEBUG:Collect: set_lang_flag: $lang=1\n" if $verbose;
    $self->{'lang'}->{$lang}=1;
}
sub clear_lang_flag {
    my ($self,$lang)=@_;
    Carp::confess "unsupported lang $_[1]" unless $known_lang{$_[1]};
    print STDERR "DEBUG:Collect: clear_lang_flag: $lang\n" if $verbose;
    delete $self->{'lang'}->{$lang};
}

sub add_raw_storage_deps {
    my $self=shift;
    my $storage_key=shift;
    my $type=shift;
    confess if $type ne 'ignore' and $type ne 'req';
    print STDERR "DEBUG:Collect: add_raw_storage_deps: $storage_key $type ",join(',',@_),"\n" if $verbose;
    push @{$self->{$storage_key.'_'.$type}}, @_;
}

sub getref_raw_storage_req {
    my $self=shift;
    my $storage_key=shift;
    return $self->{$storage_key.'_req'};
}

sub getref_raw_storage_ignore {
    my $self=shift;
    my $storage_key=shift;
    return $self->{$storage_key.'_ignore'};
}

sub add_path_req {
    my ($self)=shift;
    #&__check_arg_ref;
    print STDERR "DEBUG:Collect: add_path_req: ",join(',',@_),"\n" if $verbose;
    #push @{$self->{path_req}}, @_;
    foreach my $path (@_) {
	next unless $path;
	my @out;
	my @in=($path);
	@in=@$path if ref $path;
	local $_;
	my $before;
	foreach (@in) {
	    my $before=$_;
	    s/^["']//;s/["']$//;s/\s.*$//;s/\$EXEEXT$//;
	    Carp::carp "add_path_req: bad input [$before]: mapped to [$_]\n" if $_ ne $before;
	    push @out,$_;
	}
	if (ref $path) {
	    push @{$self->{path_req}}, \@out;
	} else {
	    push @{$self->{path_req}}, @out;
	}
    }
}

my $has_template= q!sub {
    return scalar @{$_[0]->{XXX_YYY}};
}!;

my $get_template= q!sub {
    return @{$_[0]->{XXX_YYY}};
}!;

my $getref_template= q!sub {
    return $_[0]->{XXX_YYY};
}!;

my $add_template= q!sub {
    my $self=shift;
    print STDERR 'DEBUG: Collect: add_XXX_YYY: ',join(',',@_),"\n" if $verbose;
    push @{$self->{XXX_YYY}}, @_;
}!;

foreach my $dep_key (@_DEPS_KEYS) {
    foreach my $dep_type (qw/req ignore/) {
	my $replacement=$dep_key.'_'.$dep_type;
	&create_method('has_', $has_template, $replacement);
	&create_method('get_', $get_template, $replacement);
	#&create_method('getref_', $get_template, $replacement);
	# custom definition for add_path_req
	next if $dep_key eq 'path' and $dep_type eq 'req';
	&create_method('add_', $add_template, $replacement);
    }
}

sub create_method {
    my ($subprefix,$template, $replacement)=@_;
    my $code = $template =~s/XXX_YYY/$replacement/gr;
    my $subref = eval $code;
    die 'Eval failed for ',$replacement,': ',$@,":\n", $code, "\n", unless $subref;
    {
	no strict 'refs';
	*{$subprefix.$replacement} = $subref;
    }
}

1;
