package RPM::Source::Transformation::Factory::DependencyAnalyzer::SRPMCollector;
use strict;
use warnings;
use Carp;

use File::Basename;
use Source::Bundle::Factory;
use RPM::Source::Dependency::Analyzer 0.045;
use RPM::Source::Transformation::Resource::DependencyAnalyzer::Storage;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Dependency::Analyzer::Collector;
use RPM::Source::Transformation::Factory::DependencyAnalyzer::CollectorCommon;
use RPM::Source::Transformation::Factory::DependencyAnalyzer::Resolver;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# TODO: move to AltlinuxImport?
use Source::Shared::Resource::DistroMap;

# required for srpm import
$RPM::Source::Dependency::Analyzer::Collector::opt_scan_configure_for_broken_cpp_check=1;

our $verbose=0;

our $opt_sourcenum_allow_mask;
my $opt_sourcenum_allow=[];
our $opt_sourcenum_deny_mask;
my $opt_sourcenum_deny=[];

sub cli_container_ref {
    return ['RPM::Source::Dependency::Analyzer::Collector'];
}

sub print_longopt {
    print "  SRPM Source::Dependency::Analyzer Collector options:
	--sourcedep-sourcenum-allow-mask <mask> analyze only sources <mask>
	--sourcedep-sourcenum-deny-mask <mask> do not analyze sources <mask>
	where <mask> is a set of coma-separated intervals, for example, 0,5-8
";
}

our @LONGOPT=(
    'sourcedep-sourcenum-allow-mask=s'	=> \$opt_sourcenum_allow_mask,
    'sourcedep-sourcenum-deny-mask=s'	=> \$opt_sourcenum_deny_mask,
);


sub process_options {
    if ($opt_sourcenum_allow_mask) {
	&__mask_to_array(
	    $opt_sourcenum_allow_mask,
	    $opt_sourcenum_allow,
	    "--analyzer-sourcenum-allow-mask"
	    );
    }
    if ($opt_sourcenum_deny_mask) {
	&__mask_to_array(
	    $opt_sourcenum_deny_mask,
	    $opt_sourcenum_deny,
	    "--analyzer-sourcenum-deny-mask"
	    );
    }
}

my $collector;

sub init_factory {
    my (undef,$resourceobj)=@_;
}

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_sourcenum_allow => $opt_sourcenum_allow,
	-opt_sourcenum_deny => $opt_sourcenum_deny,
    });
}


sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    #-priority => PRIORITY_PRE_BUNDLE, ??? for bundle it can make sense, though
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'distromap_source_analyzer',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_DEFAULT,
	    -name => 'distromap_source_analyzer_hack_resolve_umbiguous_pkgconfig',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig),
	unless $RPM::Source::Dependency::Analyzer::Collector::opt_disable;
    return;
}

sub opt_handler_distromap_source_analyzer {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $spec_sourcedir=$spec->{SOURCEDIR};
    unless (defined $spec_sourcedir) {
	print STDERR "WARNING: SourceAnalyzer: sourcedir is not defined. Can't find the sources.\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }
    my $name=$spec->main_section->get_tag('Name');
    if (not defined($name) and defined($macros->get_raw('name'))) {
	$name=$macros->macro_subst('%name');
    }
    unless (defined $name) {
	print STDERR "WARN: get_tag('Name') failed: something wrong with the spec?\n" if $Source::Shared::verbose;
    } elsif ($name =~ /^mingw/) {
	print STDERR "info: SourceAnalyzer: mingw support is not implemented.\n" if $Source::Shared::verbose;
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }

    my $storage=$resourceobj->dependency_analyzer_storage();
    $collector||=RPM::Source::Dependency::Analyzer::Collector->new(
	-verbose => $Source::Shared::verbose,
	# useless hint: better be hint guess from buildreq-src
	-hint => 'convert',
	-scan_configure_for_broken_cpp_check => 1,
	);
    my $is_analyzed_tarballish_source;
    my $is_failed_to_find_main_source;
    my $tarballish_re=qr/\.(?:tar|tbz|tgz|txz|zip)(?:$|\.(?:gz|bz|xz))/;
    my @raw_sourcelist_ref=@{$spec->raw_sourcelist_ref};
    my $sourcenum_allow=$opt->{-opt_sourcenum_allow};
    my $sourcenum_deny=$opt->{-opt_sourcenum_deny};
    for (my $i=0; $i<=$#raw_sourcelist_ref; $i++) {
	next if @$sourcenum_deny>0 and $sourcenum_deny->[$i];
	next if @$sourcenum_allow>0 and ! $sourcenum_allow->[$i];
	my $srcitem=$raw_sourcelist_ref[$i];
	next unless $srcitem;
	print STDERR "DistroMap::SourceAnalyzer: looking for $srcitem\n" if $verbose;
	$srcitem=$macros->macro_subst($srcitem);
	# due to something like %{download_root}MTLc3m.ttf
	my $filename=$spec_sourcedir.'/'.basename($srcitem);
	if (not -f $filename) {
	    print STDERR "DistroMap::SourceAnalyzer: no file $filename:\n" if $verbose or $Source::Shared::verbose;
	    $is_failed_to_find_main_source=1 if $i==0;
	    next;
	}
	$is_analyzed_tarballish_source=1 if $filename =~ $tarballish_re;
	&__analyze_filename($collector, $storage, $filename);
    }
    if ($is_failed_to_find_main_source and not $is_analyzed_tarballish_source) {
	my @tarballish=grep {$_ =~ $tarballish_re} glob ($spec_sourcedir.'/*.*');
	if (@tarballish==1) {
	    my $filename=$tarballish[0];
	    print STDERR "DistroMap::SourceAnalyzer: resorted to file $filename:\n" if $verbose or $Source::Shared::verbose;
	    &__analyze_filename($collector, $storage, $filename);
	}
    }

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __analyze_filename {
    my ($collector, $storage, $filename)=@_;
    my $tarball=Source::Bundle::Factory->create($filename);
    print STDERR "DistroMap::SourceAnalyzer: analyzing $filename\n" if $verbose;
    $collector->collect_source_dependencies($storage,$tarball);
}


# TODO! Distromap it! or move to SourceAnalyzer internal hack option!!
my %dep_subst_hack=qw!
pkgconfig(pygobject-2.0) python3-module-pygobject-devel
perl(mro.pm) perl-base
/usr/bin/convert ImageMagick-tools
!;

my %dep_drop_hack=map{$_=>1} qw!
/usr/bin/xemacs
!;

sub opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $buildreq = $resourceobj->buildreq;
    foreach my $dep ($buildreq->get_list_buildreq) {
	if ($dep_subst_hack{$dep}) {
	    $buildreq->del_buildreq($dep);
	    $buildreq->add_buildreq_for("replaced automatic dependency $dep", $dep_subst_hack{$dep});
	    print STDERR "info: HACK resolved dependency $dep into $dep_subst_hack{$dep}.\n" if $Source::Shared::verbose;
	} elsif ($dep_drop_hack{$dep}) {
	    $buildreq->del_buildreq($dep);
	    print STDERR "info: HACK dropped dependency $dep\n" if $Source::Shared::verbose;
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __mask_to_array {
    my ($mask,$arrayref, $msg)=@_;
    foreach my $part (split (',',$mask)) {
	if ($part=~/^(\d+)$/) {
	    $arrayref->[$1]=1;
	} elsif ($part=~/^(\d+)-(\d+)$/) {
	    my ($start,$finish)=($1,$2);
	    for (my $i=$start; $i<=$finish;$i++) {
		$arrayref->[$i]=1;
	    }
	} else {
	    die "$msg: bad part [$part] in mask $mask";
	}
    }
}

1;
